using System;
using com.epson.bank.driver;

namespace TMS1000Sample
{
	/// <summary>
	/// explanation of the outline of ApiUsage
	/// </summary>
	public class ApiUsage
	{
		protected MFDevice m_objmfDevice = null;
		protected MFBase m_objmfBase = null;
		protected MFScan m_objmfScanFront = null;
		protected MFScan m_objmfScanBack = null;
		protected MFMicr m_objmfMicr = null;
		protected MFPrint m_objmfPrint = null;
		protected MFProcess m_objmfProcess = null;
		protected ErrorCode m_errResultCode = ErrorCode.SUCCESS;
		protected TMS1000SampleForm m_objTms1000 = null;
		protected System.IAsyncResult m_IAar = null;

		UpdateCallback m_UpdateCallback = null;

		public ApiUsage()
		{
			// 
			// TODO: Add the constructor logic here
			//
			MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY);
			// Connect to the printer
			m_objmfDevice = new MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U");
			m_objmfBase = new MFBase();
			m_objmfScanFront = new MFScan();
			m_objmfScanBack = new MFScan();
			m_objmfMicr = new MFMicr();
			m_objmfPrint = new MFPrint();
			m_objmfProcess = new MFProcess();

			if(CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer\n\nEnsure Driver installed correctly\nand Printer is powered on"))
			{
				return;
			}

			// register for events
			// Register the callback destination
			m_objmfDevice.SCNMICRStatusCallback += new MFDevice.SCNMICRStatusCallbackHandler(SCNMICRSetStatusBack);
			m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack();
			CheckResponse(m_errResultCode, "SCNMICRSetStatusBack");

		}

		// this method is called when the user click the exit button
		public void Close()
		{
			if(m_objmfDevice != null)
			{
				m_objmfDevice.SCNMICRCancelStatusBack();
				m_objmfDevice.SCNMICRStatusCallback -= new MFDevice.SCNMICRStatusCallbackHandler(SCNMICRSetStatusBack);
				m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE);
				m_objmfDevice.CloseMonPrinter();
			}
		}

		// this method is called when the user click the cancel button
		public void CancelScan()
		{
			if(m_objmfDevice != null)
			{
				CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE),
								"SCNMICRCancelFunction(MF_EJECT_DISCHARGE)");
			}
		}

		// copy TMS1000SampleForm object
		public void MainFormItem(TMS1000SampleForm Item)
		{
			m_objTms1000 = Item;
		}

		// this method is return transactionNumber
		public int GetTransactionNumber()
		{
			return m_objmfDevice.TransactionNumber;
		}

		// this method configures the parameters used while scanning single checks
		// primarily the BiSCNMICRFunctionPostPrint method is used
		public void Configure()
		{
			// Base
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfBase, MF_GET_BASE_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfBase, MF_SET_BASE_PARAM)"))
			{
				return;
			}

			// ScanFront
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)"))
			{
				return;
			}

			// ScanBack
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)"))
			{
				return;
			}

			// Micr
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfMicr, MF_GET_MICR_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfMicr, MF_SET_MICR_PARAM)"))
			{
				return;
			}

			// Print
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfPrint, MF_GET_PRINT_DEFAULT)"))
			{
				return;
			}
			// An error when the lpString array is all NULL
			m_objmfPrint.String[0] = "";	// Specify blank characters
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfPrint, MF_SET_PRINT_PARAM)"))
			{
				return;
			}

			// Process
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfProcess, MF_GET_PROCESS_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfProcess, MF_SET_PROCESS_PARAM)"))
			{
				return;
			}
		}

		// This method is called when the user clicks the scan button
		// operation is offloaded to the correct Scan function
		public void Scan()
		{
			bool bRet = CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC),
								"SCNMICRFunctionPostPrint(MF_EXEC)");

			if(bRet)
			{
				m_objTms1000.Complate(0);
			}
		}

		// This method is called whenever the scan state changes
		private void SCNMICRSetStatusBack(int iTransactionNumber, MainStatus mainStatus, ErrorCode subStatus, string portName)
		{
			// Finish scan process
			if(mainStatus == MainStatus.MF_FUNCTION_DONE)
			{
				if(m_IAar != null)
				{
					m_objTms1000.EndInvoke(m_IAar);
					m_IAar = null;
				}

				// Scan End
				m_objTms1000.Complate(iTransactionNumber);
			}
			// Data receive complete
			else if(mainStatus == MainStatus.MF_DATARECEIVE_DONE)
			{
				string strMicr = "";

				// get MICR data
				// Specify the relevant item for MICR text
				m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR;
				m_objmfMicr.Parsing = false;
				// Acquire MICR text
				m_errResultCode = m_objmfDevice.GetMicrText(iTransactionNumber, m_objmfMicr);
				strMicr = m_objmfMicr.MicrStr.Substring(0, m_objmfMicr.MicrStr.IndexOf("\0"));
				if(!ErrorCode.SUCCESS.Equals(m_objmfMicr.Ret))
				{
					// Add error code
					strMicr += "(" + m_objmfMicr.Ret + ")";
				}

				// get front image
				// Set the relevant item for the front side image
				m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);
				if(ErrorCode.SUCCESS.Equals(m_errResultCode))
				{
					// Specify the file format in BMP
					m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP);
					if(ErrorCode.SUCCESS.Equals(m_errResultCode))
					{
						// Acquire image data
						m_objmfDevice.GetScanImage(iTransactionNumber, m_objmfScanFront);
					}
				}

				// get back image
				// Set the relevant item for the front side
				m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK);
				if(ErrorCode.SUCCESS.Equals(m_errResultCode))
				{
					// Specify the file format in BMP
					m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP);
					if(ErrorCode.SUCCESS.Equals(m_errResultCode))
					{
						// Acquire image data
						m_objmfDevice.GetScanImage(iTransactionNumber, m_objmfScanBack);
					}
				}

				Update(m_objmfScanFront.Image, m_objmfScanBack.Image, strMicr);
			}
		}

		// This delegate enables asynchronous calls for setting.
		delegate void UpdateCallback(System.Drawing.Image FrontImage, System.Drawing.Image BackImage, string strMicr);

		private void Update(System.Drawing.Image FrontImage, System.Drawing.Image BackImage, string strMicr)
		{
			if(m_objTms1000.InvokeRequired)
			{
				if(m_UpdateCallback == null)
				{
					m_UpdateCallback = new UpdateCallback(Update);
				}
				m_IAar = m_objTms1000.BeginInvoke(m_UpdateCallback, new object [] { FrontImage, BackImage, strMicr } );
			}
			else
			{
				m_objTms1000.SetFrontImage(FrontImage);
				m_objTms1000.SetBackImage(BackImage);
				m_objTms1000.SetMicr(strMicr);
			}
		}

		// this method is return response
		// SUCCESS : Return to FALSE
		// ERROR : Display the message box, return TRUE
		private bool CheckResponse(ErrorCode ecError, string strMethod)
		{
			if(ecError != ErrorCode.SUCCESS)
			{
				System.Windows.Forms.MessageBox.Show(strMethod, ecError.ToString());
				return true;
			}
			return false;
		}

		public TimeSpan GetStartWaitTime()
		{
			uint uiStartWaitTime = m_objmfProcess.StartWaitTime;
			if(uiStartWaitTime > 6400)
			{
				uiStartWaitTime = 6400;
			}
			return TimeSpan.FromMilliseconds((double)uiStartWaitTime);
		}
	}
}
