using System;
using com.epson.bank.driver;

namespace TMS1000Sample
{
	/// <summary>
	/// explanation of the outline of ApiUsage
	/// </summary>
	public class ApiUsage
	{
		protected MFDevice m_objmfDevice = null;
		protected MFBase m_objmfBase = null;
		protected MFScan m_objmfScanFront = null;
		protected MFScan m_objmfScanBack = null;
		protected MFMicr m_objmfMicr = null;
		protected MFPrint m_objmfPrint = null;
		protected MFProcess m_objmfProcess = null;
		protected MFIqa m_objmfIqa = null;
		protected MFBarcode m_objmfBarcodeFront = null;
		protected MFBarcode m_objmfBarcodeBack = null;
		protected ErrorCode m_errResultCode = ErrorCode.SUCCESS;
		protected TMS1000SampleForm m_objTms1000 = null;
		protected Properties m_objConfig = null;
		protected bool m_bNearFullFlg = false;
		protected ErrorCode m_errErrorOccured = ErrorCode.SUCCESS;
		protected System.Threading.WaitCallback m_NearFullThread = null;
		protected System.IAsyncResult m_IAar = null;
		UpdateCallback m_UpdateCallback = null;

		protected const ushort OCR_AB_START_X	= 30;
		protected const ushort OCR_AB_START_Y	= 56;
		protected const ushort OCR_AB_END_X		= 110;
		protected const ushort OCR_AB_END_Y		= 70;

		public ApiUsage()
		{
			// 
			// TODO: Add the constructor logic here
			//
			MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY);
			// Connect to the printer
			m_objmfDevice = new MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U");
			m_objmfBase = new MFBase();
			m_objmfScanFront = new MFScan();
			m_objmfScanBack = new MFScan();
			m_objmfMicr = new MFMicr();
			m_objmfPrint = new MFPrint();
			m_objmfProcess = new MFProcess();
			m_objmfIqa = new MFIqa();
			m_objmfBarcodeFront = new MFBarcode();
			m_objmfBarcodeBack = new MFBarcode();
			m_objConfig = new Properties();

			if(CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer\n\nEnsure Driver installed correctly\nand Printer is powered on"))
			{
				return;
			}

			// register for events
			// Register the callback destination
			m_objmfDevice.SCNMICRStatusCallback += new MFDevice.SCNMICRStatusCallbackHandler(SCNMICRSetStatusBack);
			m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack();
			CheckResponse(m_errResultCode, "SCNMICRSetStatusBack");
			m_objmfDevice.StatusCallback += new MFDevice.StatusCallbackHandler(Status);
			CheckResponse(m_objmfDevice.SetStatusBack(), "SetStatusBack");

			// 
			m_NearFullThread = new System.Threading.WaitCallback(NearFullMessage);
		}

		// this method is called when the user click the exit button
		public void Close()
		{
			if(m_objmfDevice != null)
			{
				m_objmfDevice.SCNMICRCancelStatusBack();
				m_objmfDevice.CancelStatusBack();
				m_objmfDevice.SCNMICRStatusCallback -= new MFDevice.SCNMICRStatusCallbackHandler(SCNMICRSetStatusBack);
				m_objmfDevice.StatusCallback -= new MFDevice.StatusCallbackHandler(Status);
				m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE);
				m_objmfDevice.CloseMonPrinter();
			}
		}

		// this method is called when the user click the cancel button
		public void CancelScan()
		{
			if(m_objmfDevice != null)
			{
				CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE),
								"SCNMICRCancelFunction(MF_EJECT_DISCHARGE)");
			}
		}

		// copy TMS1000SampleForm object
		public void MainFormItem(TMS1000SampleForm Item)
		{
			m_objTms1000 = Item;
		}

		// this method is return transactionNumber
		public int GetTransactionNumber()
		{
			return m_objmfDevice.TransactionNumber;
		}

		// This method is called when the program starts or the user changes configuration
		// operation is offloaded to the correct ConfigureXxx function
		public void Configure(Properties Proc)
		{
			// DPI setting method
			// Specify MF_SCAN_DPI_XXX in sResolution of the MF_SCAN structure
			// Execute BiSCNMICRFunctionXXX
			// * Execute before scanning

			// Sharpening setting method
			// Specify EPS_BI_SCN_XXX as the 5th parameter of BiSCNSetImageQuality and 	execute it (other parameters are discretionary)
			// * Execute before scanning

			// Setting method for multiple electronic endorsements
			// Execute MF_PRT_BUFFERING with BiBufferedPrint
			// Set multiple electronic endorsements
			// Execute MF_PRT_EXEC with BiBufferedPrint
			// * Execute before scanning

			
			SetProc(Proc);

			// Set station electronic endorse
			if(CheckResponse(m_objmfDevice.SetPrintStation(PrintingStation.MF_ST_E_ENDORSEMENT), "SetPrintStation(MF_ST_E_ENDORSEMENT)"))
			{
				return;
			}

			// the registered electronic endorsement
			if(CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_CLEAR), "BufferedPrint(MF_PRT_CLEAR)"))
			{
				return;
			}

			switch(m_objConfig.GetValue(Properties.SCAN_FUNC))
			{
				case 0:
					ConfigureMulti();
					break;
				case 1:
					ConfigureSingle();
					break;
				default:
					ConfigureMulti();
					break;
			}

			WaterfallMode eWaterfallMode = WaterfallMode.WATERFALL_MODE_DISABLE;
			if(m_objConfig.GetValueBool(Properties.WATERFALL_ENABLE))
			{
				if(m_objConfig.GetValue(Properties.WATERFALL_MODE) == 0)
				{
					eWaterfallMode = WaterfallMode.WATERFALL_MODE_STANDARD;
				}
				else
				{
					eWaterfallMode = WaterfallMode.WATERFALL_MODE_INHERIT_POCKET;
				}
			}
			// Set Waterfall mode
			if(CheckResponse(m_objmfDevice.SetWaterfallMode(eWaterfallMode), "SetWaterfallMode(eWaterfallMode)"))
			{
				return;
			}
		}

		// this method is settings of multi scan
		// primarily the BiSCNMICRFunctionContinuously method is used
		private void ConfigureMulti()
		{
			// Base
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfBase, MF_GET_BASE_DEFAULT)"))
			{
				return;
			}
			SetBaseStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_SET_BASE_PARAM),
				"SCNMICRFunctionContinuously(m_objmfBase, MF_SET_BASE_PARAM)"))
			{
				return;
			}

			// ScanFront
			if(m_objConfig.GetValueBool(Properties.SCAN_FRONT))
			{
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT),
					"SCNMICRFunctionContinuously(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)"))
				{
					return;
				}
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM),
					"SCNMICRFunctionContinuously(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)"))
				{
					return;
				}
			}
			else
			{
				// Delete the structure set in MF_CLEAR_SCAN_FRONT_PARAM when the front side will not be scanned
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_CLEAR_SCAN_FRONT_PARAM),
					"SCNMICRFunctionContinuously(mfScanFount, MF_CLEAR_SCAN_FRONT_PARAM)"))
				{
					return;
				}
			}

			// ScanBack
			if(m_objConfig.GetValueBool(Properties.SCAN_BACK))
			{
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT),
					"SCNMICRFunctionContinuously(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)"))
				{
					return;
				}
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM),
					"SCNMICRFunctionContinuously(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)"))
				{
					return;
				}
			}
			else
			{
				// Delete the structure set in MF_CLEAR_SCAN_BACK_PARAM when the reverse side will not be scanned
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_CLEAR_SCAN_BACK_PARAM),
					"SCNMICRFunctionContinuously(m_objmfScanBack, MF_CLEAR_SCAN_BACK_PARAM)"))
				{
					return;
				}
			}

			// Micr
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfMicr, MF_GET_MICR_DEFAULT)"))
			{
				return;
			}
			// MICR font designation
			switch(m_objConfig.GetValue(Properties.MICR_FONT))
			{
				case 0:
					m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_E13B;
					break;
				case 1:
					m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_CMC7;
					break;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM),
				"SCNMICRFunctionContinuously(m_objmfMicr, MF_SET_MICR_PARAM)"))
			{
				return;
			}

			// Print
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfPrint, MF_GET_PRINT_DEFAULT)"))
			{
				return;
			}
			// An error when the lpString array is all NULL
			m_objmfPrint.String[0] = "";	// Specify blank characters
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM),
				"SCNMICRFunctionContinuously(m_objmfPrint, MF_SET_PRINT_PARAM)"))
			{
				return;
			}

			// Operational settings when an error occurs
			// Process
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfProcess, MF_GET_PROCESS_DEFAULT)"))
			{
				return;
			}
			SetProcessStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM),
				"SCNMICRFunctionContinuously(m_objmfProcess, MF_SET_PROCESS_PARAM)"))
			{
				return;
			}	

			// Set the endorsement
			if(m_objConfig.GetValueBool(Properties.ELEC_ENDORSE_IMAGE))
			{
				// Set the electronic endorsement
				if(CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), 
					"BufferedPrint(MF_PRT_BUFFERING)"))
				{
					return;
				}

				// Set the image electronic endorsement
				ElectricEndorseImage();

				// Register the electronic endorsement set
				if(CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC),
					"BufferedPrint(MF_PRT_EXEC)"))
				{
					return;
				}
			}

			// Iqa
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfIqa, FunctionType.MF_GET_IQA_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfIqa, MF_GET_IQA_DEFAULT)"))
			{
				return;
			}
			SetIqaStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfIqa, FunctionType.MF_SET_IQA_PARAM),
				"SCNMICRFunctionContinuously(m_objmfIqa, MF_SET_IQA_PARAM)"))
			{
				return;
			}

			// Barcode
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeFront, FunctionType.MF_GET_BARCODE_FRONT_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfBarcodeFront, MF_GET_BARCODE_FRONT_DEFAULT)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeBack, FunctionType.MF_GET_BARCODE_BACK_DEFAULT),
				"SCNMICRFunctionContinuously(m_objmfBarcodeBack, MF_GET_BARCODE_BACK_DEFAULT)"))
			{
				return;
			}
			SetBarcodeStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeFront, FunctionType.MF_SET_BARCODE_FRONT_PARAM),
				"SCNMICRFunctionContinuously(m_objmfBarcodeFront, MF_SET_BARCODE_FRONT_PARAM)"))
			{
				return;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeBack, FunctionType.MF_SET_BARCODE_BACK_PARAM),
				"SCNMICRFunctionContinuously(m_objmfBarcodeBack, MF_SET_BARCODE_BACK_PARAM)"))
			{
				return;
			}

		}

		// this method is settings of single scan
		// primarily the BiSCNMICRFunctionPostPrint method is used
		private void ConfigureSingle()
		{
			// Base
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfBase, MF_GET_BASE_DEFAULT)"))
			{
				return;
			}
			SetBaseStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfBase, MF_SET_BASE_PARAM)"))
			{
				return;
			}

			// ScanFront
			if(m_objConfig.GetValueBool(Properties.SCAN_FRONT))
			{
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT),
					"SCNMICRFunctionPostPrint(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)"))
				{
					return;
				}
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM),
					"SCNMICRFunctionPostPrint(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)"))
				{
					return;
				}
			}
			else
			{
				// Delete the structure set in MF_CLEAR_SCAN_FRONT_PARAM when the front side will not be scanned
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_CLEAR_SCAN_FRONT_PARAM),
					"SCNMICRFunctionPostPrint(m_objmfScanFront, MF_CLEAR_SCAN_FRONT_PARAM)"))
				{
					return;
				}
			}

			// ScanBack
			if(m_objConfig.GetValueBool(Properties.SCAN_BACK))
			{
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT),
					"SCNMICRFunctionPostPrint(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)"))
				{
					return;
				}
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM),
					"SCNMICRFunctionPostPrint(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)"))
				{
					return;
				}
			}
			else
			{
				// Delete the structure set in MF_CLEAR_SCAN_BACK_PARAM when the reverse side will not be scanned
				if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_CLEAR_SCAN_BACK_PARAM),
					"SCNMICRFunctionPostPrint(m_objmfScanBack, MF_CLEAR_SCAN_BACK_PARAM)"))
				{
					return;
				}
			}

			// Micr
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfMicr, MF_GET_MICR_DEFAULT)"))
			{
				return;
			}
			// MICR font designation
			switch(m_objConfig.GetValue(Properties.MICR_FONT))
			{
				case 0:
					m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_E13B;
					break;
				case 1:
					m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_CMC7;
					break;
			}
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfMicr, MF_SET_MICR_PARAM)"))
			{
				return;
			}

			// Print
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfPrint, MF_GET_PRINT_DEFAULT)"))
			{
				return;
			}
			// An error when the lpString array is all NULL
			m_objmfPrint.String[0] = "";	// Specify blank characters
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfPrint, MF_SET_PRINT_PARAM)"))
			{
				return;
			}

			// Operational settings when an error occurs
			// Process
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfProcess, MF_GET_PROCESS_DEFAULT)"))
			{
				return;
			}
			SetProcessStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfProcess, MF_SET_PROCESS_PARAM)"))
			{
				return;
			}		
	
			// Set the endorsement
			if(m_objConfig.GetValueBool(Properties.ELEC_ENDORSE_IMAGE))
			{
				// Set the electronic endorsement
				if(CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), 
					"BufferedPrint(MF_PRT_BUFFERING)"))
				{
					return;
				}

				// Set the image electronic endorsement
				ElectricEndorseImage();

				// Register the electronic endorsement set
				if(CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC),
					"BufferedPrint(MF_PRT_EXEC)"))
				{
					return;
				}
			}

			// Iqa
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfIqa, FunctionType.MF_GET_IQA_DEFAULT),
				"SCNMICRFunctionPostPrint(m_objmfIqa, MF_GET_IQA_DEFAULT)"))
			{
				return;
			}
			SetIqaStruct();
			if(CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfIqa, FunctionType.MF_SET_IQA_PARAM),
				"SCNMICRFunctionPostPrint(m_objmfIqa, MF_SET_IQA_PARAM)"))
			{
				return;
			}	
		}

		// This method is called when the user clicks the scan button
		// operation is offloaded to the correct Scan function
		public void Scan()
		{
			m_bNearFullFlg = false;
			bool bRet = false;
			switch(m_objConfig.GetValue(Properties.SCAN_FUNC))
			{
				case 0:
					bRet = CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC),
						"SCNMICRFunctionContinuously(MF_EXEC)");
					break;
				case 1:
					bRet = CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC),
						"SCNMICRFunctionPostPrint(MF_EXEC)");
					break;
				default:
					bRet = CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC),
						"SCNMICRFunctionContinuously(MF_EXEC)");
					break;
			}

			// Failure to the scan start
			if(bRet)
			{
				m_objTms1000.Complate(0);
			}
											
		}

		// this method is insert proc in m_objConfig
		public void SetProc(Properties Proc)
		{
			m_objConfig = Proc;
		}

		// This delegate enables asynchronous calls for setting.
		delegate void RemoveMessageCallback();

		// Display Remove message
		public void RemoveMessage()
		{
			if(m_objTms1000.InvokeRequired)
			{
				RemoveMessageCallback c = new RemoveMessageCallback(RemoveMessage);
				m_objTms1000.Invoke(c, new object [] {});
			}
			else
			{
				// Display the prompt "Remove the paper".
				System.Windows.Forms.MessageBox.Show(m_objTms1000,"Remove the paper","MECHANICAL ERROR");
			}
		}

		// This delegate enables asynchronous calls for setting.
		delegate void NearFullMessageCallback(object status);

		// Display NearFull message
		public void NearFullMessage(object status)
		{
			if(m_objTms1000.InvokeRequired)
			{
				NearFullMessageCallback c = new NearFullMessageCallback(NearFullMessage);
				m_objTms1000.Invoke(c, new object [] {status});
			}
			else
			{
				if(m_bNearFullFlg)
				{
					m_objmfDevice.RingBuzzer(MfBuzzerTone.MF_BUZZER_TONE_HIGH, 3, 1, 0);
					m_objTms1000.Text = "TM-S1000SampleStep8  -  ***** NearFull *****";
				}
				else
				{
					m_objTms1000.Text = "TM-S1000SampleStep8";
				}
			}
		}

		// This method is called whenever the scan state changes
		// Called when scanning with BiSCNMICRFunctionPostPrint/BiSCNMICRFunctionContinuously
		private void SCNMICRSetStatusBack(int iTransactionNumber, MainStatus mainStatus, ErrorCode subStatus, string portName)
		{
			// Finish scan process
			if(mainStatus == MainStatus.MF_FUNCTION_DONE)
			{
				if(m_IAar != null)
				{
					m_objTms1000.EndInvoke(m_IAar);
					m_IAar = null;
				}

				// Scan End
				m_objTms1000.Complate(iTransactionNumber);

				// Mechanical Error
				if(subStatus == ErrorCode.ERR_PAPER_JAM)
				{
					RemoveMessage();

					// Cancel mechanical error
					m_objmfDevice.CancelError();
				}
				else if(subStatus != ErrorCode.SUCCESS)
				{
					System.Windows.Forms.MessageBox.Show(subStatus.ToString(), "Error");
				}
			}
				// Error Occured
			else if(mainStatus == MainStatus.MF_ERROR_OCCURED)
			{
				m_errErrorOccured = subStatus;
			}

				// NearFull
			else if(mainStatus == MainStatus.MF_CHECKPAPER_PROCESS_DONE)
			{
				ASB Status = GetStatus();
				if(!m_bNearFullFlg)
				{
					// Near full error
					if(!Convert.ToBoolean(Status & ASB.ASB_MAIN_NEAR_FULL) || !Convert.ToBoolean(Status & ASB.ASB_SUB_NEAR_FULL))
					{
						m_bNearFullFlg = true;
						System.Threading.ThreadPool.QueueUserWorkItem(m_NearFullThread);
					}
				}
				else
				{
					// No Near full error
					if(Convert.ToBoolean(Status & ASB.ASB_MAIN_NEAR_FULL) && Convert.ToBoolean(Status & ASB.ASB_SUB_NEAR_FULL))
					{
						m_bNearFullFlg = false;
						System.Threading.ThreadPool.QueueUserWorkItem(m_NearFullThread);
					}
				}
			}
				// Data receive complete
			else if(mainStatus == MainStatus.MF_DATARECEIVE_DONE)
			{
				string strMicr = "";
				string strOcrAb = "";
				bool bFrontDisplay = false;
				bool bFrontGrayscale = false;
				bool bBackDisplay = false;
				bool bBackGrayscale = false;

				// get MICR data
				if(m_objConfig.GetValueBool(Properties.MICR))
				{
					// Specify the relevant item for MICR text
					m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR;
					m_objmfMicr.Parsing = false;
					// Acquire MICR text
					m_errResultCode = m_objmfDevice.GetMicrText(iTransactionNumber, m_objmfMicr);
					strMicr = m_objmfMicr.MicrStr.Substring(0, m_objmfMicr.MicrStr.IndexOf("\0"));
					if(!ErrorCode.SUCCESS.Equals(m_objmfMicr.Ret))
					{
						// Add error code
						strMicr += "(" + m_objmfMicr.Ret + ")";
					}
				}

				if(m_objConfig.GetValueBool(Properties.OCR_AB) && m_objConfig.GetValueBool(Properties.SCAN_FRONT))
				{
					// OCR AB
					MFOcrAb mfOcrAb = new MFOcrAb();
					// Font designation
					if(m_objConfig.GetValue(Properties.OCR_AB_FONT) == 0)
					{
						mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRA_ALPHANUM;
						strOcrAb = "OCR_A:";
					}
					else
					{
						mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRB_ALPHANUM;
						strOcrAb = "OCR_B:";
					}
					mfOcrAb.StartX = OCR_AB_START_X;
					mfOcrAb.StartY = OCR_AB_START_Y;
					mfOcrAb.EndX = OCR_AB_END_X;
					mfOcrAb.EndY = OCR_AB_END_Y;
					mfOcrAb.Direction = MfDirection.MF_OCR_LEFTRIGHT;
					mfOcrAb.SpeceHandling = MfSpeceHandling.OCR_SPACE_ENABLE;

					// select front side image
					m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);
					if(ErrorCode.SUCCESS.Equals(m_errResultCode))
					{
						// Acquire OCR_AB text
						m_errResultCode = m_objmfDevice.GetOcrABText(Convert.ToUInt32(iTransactionNumber), OcrImageSource.OCR_SOURCE_TRANSACTION_NUMBER, "", mfOcrAb);
						strOcrAb += mfOcrAb.OcrStr.Substring(0, mfOcrAb.OcrStr.IndexOf("\0"));
						if(!ErrorCode.SUCCESS.Equals(mfOcrAb.Ret))
						{
							// Add error code
							strOcrAb += " ( " + mfOcrAb.Ret.ToString() + " ) ";
						}
					}
				}

				// save MICR data
				if(m_objConfig.GetValueBool(Properties.MICR_SAVE_ENABLE) && m_objConfig.GetValueBool(Properties.MICR_SAVE))
				{
					SaveMicr(iTransactionNumber);
				}

				// get front image
				if(m_objConfig.GetValueBool(Properties.SCAN_FRONT))
				{
					// Set the relevant item for the front side image
					m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);
					if(ErrorCode.SUCCESS.Equals(m_errResultCode))
					{
						// Set grayscale
						ColorDepth cdColor;
						bFrontGrayscale = m_objConfig.GetValueBool(Properties.FRONT_GRAYSCALE);
						if(bFrontGrayscale)
						{
							cdColor = ColorDepth.EPS_BI_SCN_8BIT;
						}
						else
						{
							cdColor = ColorDepth.EPS_BI_SCN_1BIT;
						}
						m_errResultCode = m_objmfDevice.SCNSetImageQuality(cdColor, 0, Color.EPS_BI_SCN_MONOCHROME, ExOption.EPS_BI_SCN_SHARP);

						// set format
						if(bFrontGrayscale)
						{
							m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_JPEGNORMAL);
						}
						else
						{
							m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_TIFF);
						}

						// Acquire image data
						m_errResultCode = m_objmfDevice.GetScanImage(iTransactionNumber, m_objmfScanFront);

						// display front image
						bFrontDisplay = m_objConfig.GetValueBool(Properties.FRONT_DISPLAY);

						// save front image
						if(m_objConfig.GetValueBool(Properties.FRONT_SAVE))
						{
							// Set grayscale
							string strFileName = "";
							if(bFrontGrayscale)
							{
								strFileName = iTransactionNumber.ToString("000") + ".front.jpg";
							}
							else
							{
								strFileName = iTransactionNumber.ToString("000") + ".front.tif";
							}

							if(ErrorCode.SUCCESS.Equals(m_errResultCode))
							{
								System.IO.Stream data = m_objmfScanFront.Data;
								System.IO.FileStream file = new System.IO.FileStream(strFileName, System.IO.FileMode.Create);
								byte [] buffer = new byte[data.Length];

								// Save in file
								data.Read(buffer, 0, buffer.Length);
								file.Write(buffer, 0, buffer.Length);

								data.Close();
								file.Close();
							}
						}
					}
				}

				// set electric endorse
				if(m_objConfig.GetValueBool(Properties.ELEC_ENDORSE_TEXT))
				{
					ElectricEndorseText();
				}

				// get back image
				if(m_objConfig.GetValueBool(Properties.SCAN_BACK))
				{
					// Set the relevant item for the front side
					m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK);
					if(ErrorCode.SUCCESS.Equals(m_errResultCode))
					{
						// Set grayscale
						ColorDepth bColor;
						bBackGrayscale = m_objConfig.GetValueBool(Properties.BACK_GRAYSCALE);
						if(bBackGrayscale)
						{
							bColor = ColorDepth.EPS_BI_SCN_8BIT;
						}
						else
						{
							bColor = ColorDepth.EPS_BI_SCN_1BIT;
						}
						m_errResultCode = m_objmfDevice.SCNSetImageQuality(bColor, 0, Color.EPS_BI_SCN_MONOCHROME, ExOption.EPS_BI_SCN_SHARP);

						// set format
						if(bFrontGrayscale)
						{
							m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_JPEGNORMAL);
						}
						else
						{
							m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_TIFF);
						}

						// Acquire image data
						m_errResultCode = m_objmfDevice.GetScanImage(iTransactionNumber, m_objmfScanBack);

						// display back image
						bBackDisplay = m_objConfig.GetValueBool(Properties.BACK_DISPLAY);

						// save back image
						if(m_objConfig.GetValueBool(Properties.BACK_SAVE))
						{
							// Set grayscale
							string strFileName = "";
							if(m_objConfig.GetValueBool(Properties.BACK_GRAYSCALE))
							{
								strFileName = iTransactionNumber.ToString("000") + ".back.jpg";
							}
							else
							{
								strFileName = iTransactionNumber.ToString("000") + ".back.tif";
							}

							if(ErrorCode.SUCCESS.Equals(m_errResultCode))
							{
								System.IO.Stream data = m_objmfScanBack.Data;
								System.IO.FileStream file = new System.IO.FileStream(strFileName, System.IO.FileMode.Create);
								byte [] buffer = new byte[data.Length];

								// Save in file
								data.Read(buffer, 0, buffer.Length);
								file.Write(buffer, 0, buffer.Length);

								data.Close();
								file.Close();
							}
						}
					}
				}

				Update(bFrontDisplay? m_objmfScanFront.Image : null, bBackDisplay? m_objmfScanBack.Image : null, strMicr, strOcrAb);

				if(m_objConfig.GetValueBool(Properties.IQA_DETECT))
				{
					MFIqaResult mfIqaResult = new MFIqaResult();
					m_objmfDevice.GetIqaResult(iTransactionNumber, mfIqaResult);
					m_objTms1000.SetIqaResult(mfIqaResult);
				}

				// display barcode text
				DisplayBarcodeData(iTransactionNumber);

				// Is it Confirmation Mode?
				if(m_objConfig.GetValueBool(Properties.CONFIRMATION))
				{
					// Execute BiSetBehaviorToScnResult?
					if(m_objConfig.GetValueBool(Properties.RUN_SCN_TO_RESULT))
					{
						// Call BiSetBehaviorToScnResult?
						if(!m_objConfig.GetValueBool(Properties.NO_CALL_SCN_TO_RESULT))
						{
							// Display BiSetBehaviorToScnResult dialog box
							// Do no execute if it is canceled in the dialog box
							if(m_objTms1000.Confirmation())
							{
								MfProcessContinue bNextCheck;

								switch(m_objConfig.GetValue(Properties.CONF_NEXT_CHECK))
								{
									case 0:
										bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP;
										break;
									case 1:
										bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_NOOVERLAP;
										break;
									case 2:
										bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_CANCEL;
										break;
									default:
										bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP;
										break;
								}

								m_objmfDevice.SetBehaviorToScnResult(GetEject(m_objConfig.GetValue(Properties.CONF_EJECT)),
									GetStamp(m_objConfig.GetValueBool(Properties.CONF_STAMP)), 
									bNextCheck);
							}
						}
						else
						{
							// Use the value set without calling BiSetBehaviorToScnResult

							MfProcessContinue bNextCheck;

							switch(m_objConfig.GetValue(Properties.CONF_NEXT_CHECK))
							{
								case 0:
									bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP;
									break;
								case 1:
									bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_NOOVERLAP;
									break;
								case 2:
									bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_CANCEL;
									break;
								default:
									bNextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP;
									break;
							}

							m_objmfDevice.SetBehaviorToScnResult(GetEject(m_objConfig.GetValue(Properties.CONF_EJECT)),
								GetStamp(m_objConfig.GetValueBool(Properties.CONF_STAMP)), 
								bNextCheck);
						}
					}
				}
				m_errErrorOccured = ErrorCode.SUCCESS;
			}
		}

		// This delegate enables asynchronous calls for setting.
		delegate void UpdateCallback(System.Drawing.Image FrontImage, System.Drawing.Image BackImage, string strMicr, string strOcrAb);

		private void Update(System.Drawing.Image FrontImage, System.Drawing.Image BackImage, string strMicr, string strOcrAb)
		{
			if(m_objTms1000.InvokeRequired)
			{
				if(m_UpdateCallback == null)
				{
					m_UpdateCallback = new UpdateCallback(Update);
				}
				m_IAar = m_objTms1000.BeginInvoke(m_UpdateCallback, new object [] { FrontImage, BackImage, strMicr, strOcrAb } );
			}
			else
			{
				m_objTms1000.SetFrontImage(FrontImage);
				m_objTms1000.SetBackImage(BackImage);
				m_objTms1000.SetMicr(strMicr);
				m_objTms1000.SetOcr(strOcrAb);
			}
		}

		// settings endorse text
		private void ElectricEndorseText()
		{
			MFTrueType mfDecorate = null;
			System.Drawing.Font objFont = null;

			// set position
			if(CheckResponse(m_objmfDevice.SetPrintPosition(10, 60), "SetPrintPosition(10, 60)"))
			{
				return;
			}

			mfDecorate = new MFTrueType();
			mfDecorate.Color = PrintColor.MF_PRINT_BLACK;
			objFont = new System.Drawing.Font("Arial", 30);
			mfDecorate.Font = objFont;
			m_errResultCode = m_objmfDevice.PrintText("Transaction#:<00000000>", mfDecorate);
			CheckResponse(m_errResultCode, "BiPrintText");
		}

		// settings endorse image
		private void ElectricEndorseImage()
		{
			// set position
			if(CheckResponse(m_objmfDevice.SetPrintPosition(100, 30), "SetPrintPosition(100, 30)"))
			{
				return;
			}

			// set size
			if(CheckResponse(m_objmfDevice.SetPrintSize(30, 30), "SetPrintSize(30, 30)"))
			{
				return;
			}

			// set endowse image
			CheckResponse(m_objmfDevice.PrintImage("image.jpg"), "PrintImage(image.jpg)");
		}

		// this method is return response
		// SUCCESS : Return to FALSE
		// ERROR : Display the message box, return TRUE
		private bool CheckResponse(ErrorCode ecError, string strMethod)
		{
			if(ecError != ErrorCode.SUCCESS)
			{
				System.Windows.Forms.MessageBox.Show(strMethod, ecError.ToString());
				return true;
			}
			return false;
		}
		
		// this method is return the  value of MF_BUZZER_HZ_XXX
		private MfBuzzerHz BuzzerHz(int nValue)
		{
			switch(nValue)
			{
				case 0:
					return MfBuzzerHz.MF_BUZZER_HZ_440;
				case 1:
					return MfBuzzerHz.MF_BUZZER_HZ_880;
				case 2:
					return MfBuzzerHz.MF_BUZZER_HZ_4000;
				default:
					return MfBuzzerHz.MF_BUZZER_HZ_440;
			}
		}

		// this method is return the  value of MF_BUZZER_COUNT_XXX
		private MfBuzzerCount BUzzerCount(int nValue)
		{
			switch(nValue)
			{
				case 0:
					return MfBuzzerCount.MF_BUZZER_DISABLE;
				case 1:
					return MfBuzzerCount.MF_BUZZER_COUNT_ONE;
				case 2:
					return MfBuzzerCount.MF_BUZZER_COUNT_TWO;
				case 3:
					return MfBuzzerCount.MF_BUZZER_COUNT_MAX;
				default:
					return MfBuzzerCount.MF_BUZZER_DISABLE;
			}
		}

		// settings Base structure
		private void SetBaseStruct()
		{
			m_objmfBase.BuzzerHz[MfBuzzerType.MF_BUZZER_TYPE_SUCCESS] = BuzzerHz(m_objConfig.GetValue(Properties.BUZZER_SUCCESS_HZ));
			m_objmfBase.BuzzerCount[MfBuzzerType.MF_BUZZER_TYPE_SUCCESS] = BUzzerCount(m_objConfig.GetValue(Properties.BUZZER_SUCCESS_COUNT));

			m_objmfBase.BuzzerHz[MfBuzzerType.MF_BUZZER_TYPE_ERROR] = BuzzerHz(m_objConfig.GetValue(Properties.BUZZER_ERROR_HZ));
			m_objmfBase.BuzzerCount[MfBuzzerType.MF_BUZZER_TYPE_ERROR] = BUzzerCount(m_objConfig.GetValue(Properties.BUZZER_ERROR_COUNT));

			m_objmfBase.BuzzerHz[MfBuzzerType.MF_BUZZER_TYPE_WFEED] = BuzzerHz(m_objConfig.GetValue(Properties.BUZZER_WFEED_HZ));
			m_objmfBase.BuzzerCount[MfBuzzerType.MF_BUZZER_TYPE_WFEED] = BUzzerCount(m_objConfig.GetValue(Properties.BUZZER_WFEED_COUNT));
		}
		
		// settings Process structure
		private void SetProcessStruct()
		{
			// ActivationMode
			if(m_objConfig.GetValueBool(Properties.CONFIRMATION))
			{
				m_objmfProcess.ActivationMode = MfActivateMode.MF_ACTIVATE_MODE_CONFIRMATION;
			}
			else
			{
				m_objmfProcess.ActivationMode = MfActivateMode.MF_ACTIVATE_MODE_HIGH_SPEED;
			}

			// PaperMisInsertion
			m_objmfProcess.PaperMisInsertionErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.MIS_INSERT_DETECT));
			m_objmfProcess.PaperMisInsertionErrorEject = GetEject(m_objConfig.GetValue(Properties.MIS_INSERT_EJECT));
			m_objmfProcess.PaperMisInsertionStamp = GetStamp(m_objConfig.GetValueBool(Properties.MIS_INSERT_STAMP));
			m_objmfProcess.PaperMisInsertionCancel = GetCancel(m_objConfig.GetValueBool(Properties.MIS_INSERT_CANCEL));

			// Noise
			m_objmfProcess.NoiseErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.NOISE_DETECT));
			m_objmfProcess.NoiseErrorEject = GetEject(m_objConfig.GetValue(Properties.NOISE_EJECT));
			m_objmfProcess.NoiseStamp = GetStamp(m_objConfig.GetValueBool(Properties.NOISE_STAMP));
			m_objmfProcess.NoiseCancel = GetCancel(m_objConfig.GetValueBool(Properties.NOISE_CANCEL));

			// DoubleFeed
			m_objmfProcess.DoubleFeedErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.DOUBLE_FEED_DETECT));
			m_objmfProcess.DoubleFeedErrorEject = GetEject(m_objConfig.GetValue(Properties.DOUBLE_FEED_EJECT));
			m_objmfProcess.DoubleFeedStamp = GetStamp(m_objConfig.GetValueBool(Properties.DOUBLE_FEED_STAMP));
			m_objmfProcess.DoubleFeedCancel = GetCancel(m_objConfig.GetValueBool(Properties.DOUBLE_FEED_CANCEL));

			// Baddata
			m_objmfProcess.BaddataCount = Convert.ToByte(m_objConfig.GetValue(Properties.BADDATA_COUNT));
			m_objmfProcess.BaddataErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.BADDATA_DETECT));
			m_objmfProcess.BaddataErrorEject = GetEject(m_objConfig.GetValue(Properties.BADDATA_EJECT));
			m_objmfProcess.BaddataStamp = GetStamp(m_objConfig.GetValueBool(Properties.BADDATA_STAMP));
			m_objmfProcess.BaddataCancel = GetCancel(m_objConfig.GetValueBool(Properties.BADDATA_CANCEL));

			// Nodata
			m_objmfProcess.NodataErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.NODATA_DETECT));
			m_objmfProcess.NodataErrorEject = GetEject(m_objConfig.GetValue(Properties.NODATA_EJECT));
			m_objmfProcess.NodataStamp = GetStamp(m_objConfig.GetValueBool(Properties.NODATA_STAMP));
			m_objmfProcess.NodataCancel = GetCancel(m_objConfig.GetValueBool(Properties.NODATA_CANCEL));

			// NearFull
			switch(m_objConfig.GetValue(Properties.NEAR_FULL))
			{
				case 0:
					m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_PERMIT;
					break;
				case 1:
					m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_MAIN_PERMIT;
					break;
				case 2:
					m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_SUB_PERMIT;
					break;
				case 3:
					m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_NOT_PERMIT;
					break;
			}

			// Success Stamp
			m_objmfProcess.SuccessStamp = GetStamp(m_objConfig.GetValueBool(Properties.SUCCESS_STAMP));
		}

		// settings Iqa structure
		private void SetIqaStruct()
		{
			m_objmfIqa.ErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.IQA_DETECT));
			m_objmfIqa.ErrorEject = GetEject(m_objConfig.GetValue(Properties.IQA_EJECT));
			m_objmfIqa.Stamp = GetStamp(m_objConfig.GetValueBool(Properties.IQA_STAMP));
			m_objmfIqa.Cancel = GetCancel(m_objConfig.GetValueBool(Properties.IQA_CANCEL));

			if(m_objConfig.GetValueBool(Properties.IQA_GRAYSCALE))
			{
				m_objmfIqa.ColorDepth = ColorDepth.EPS_BI_SCN_8BIT;
				m_objmfIqa.ImageFormat = Format.EPS_BI_SCN_JPEGNORMAL;
			}
			else
			{
				m_objmfIqa.ColorDepth = ColorDepth.EPS_BI_SCN_1BIT;
				m_objmfIqa.ImageFormat = Format.EPS_BI_SCN_TIFF;
			}

			if(m_objmfIqa.ErrorSelect == MfErrorSelect.MF_ERROR_SELECT_DETECT)
			{
				m_objmfIqa.Undersize = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Oversize = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Mincompressed = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Maxcompressed = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Front_rear = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Toolight = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Toodark = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Streaks = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Noise = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Focus = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Corners = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Edges = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Framing = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Skew = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Carbon = MfIqaTest.MF_IQA_TEST_ENABLE;
				m_objmfIqa.Piggyback = MfIqaTest.MF_IQA_TEST_ENABLE;
			}
		}

		// setting Barcode structure
		private void SetBarcodeStruct()
		{
			m_objmfBarcodeFront.ErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.BARCODE_FRONT_DETECT));
			m_objmfBarcodeFront.ErrorEject = GetEject(m_objConfig.GetValue(Properties.BARCODE_FRONT_EJECT));
			m_objmfBarcodeFront.Stamp = GetStamp(m_objConfig.GetValueBool(Properties.BARCODE_FRONT_STAMP));
			m_objmfBarcodeFront.Cancel = GetCancel(m_objConfig.GetValueBool(Properties.BARCODE_FRONT_CANCEL));

			m_objmfBarcodeFront.Info[0].SymbolMask = (
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODABAR |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE128 |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE39 |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_EAN_JAN |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_ITF |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_A |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_E);

			m_objmfBarcodeBack.ErrorSelect = GetDetect(m_objConfig.GetValueBool(Properties.BARCODE_BACK_DETECT));
			m_objmfBarcodeBack.ErrorEject = GetEject(m_objConfig.GetValue(Properties.BARCODE_BACK_EJECT));
			m_objmfBarcodeBack.Stamp = GetStamp(m_objConfig.GetValueBool(Properties.BARCODE_BACK_STAMP));
			m_objmfBarcodeBack.Cancel = GetCancel(m_objConfig.GetValueBool(Properties.BARCODE_BACK_CANCEL));

			m_objmfBarcodeBack.Info[0].SymbolMask = (
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODABAR |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE128 |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE39 |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_EAN_JAN |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_ITF |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_A |
				MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_E);
		}

		// MICR data is preserved
		private void SaveMicr(int transactionNumber)
		{
			// Create log file name
			System.IO.StreamWriter sw = new System.IO.StreamWriter(transactionNumber.ToString() + ".Micr.txt");
			try
			{
				// Values that are always printed out
				sw.WriteLine("# MICR Data Output");
				sw.WriteLine();
				sw.WriteLine("# General Transaction Information:");
				sw.WriteLine();
				sw.WriteLine("TransactionNumber = " + transactionNumber.ToString());
				// MICR
				if(m_objConfig.GetValueBool(Properties.MICR))
				{
					m_objmfMicr.Parsing = true;
					// Acquire MICR text
					m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR;
					// Get the text acquired
					m_objmfDevice.GetMicrText(transactionNumber, m_objmfMicr);
					sw.WriteLine("Result = " + m_objmfMicr.Ret.ToString());
					sw.WriteLine("Status = " + m_objmfMicr.Status.ToString());
					sw.WriteLine("Detail = " + m_objmfMicr.Detail.ToString());
					sw.WriteLine();
					sw.WriteLine();
					sw.WriteLine("# Magnetic Character Results:");
					sw.WriteLine();
					sw.WriteLine("Raw Data       = " + m_objmfMicr.MicrStr);
					sw.WriteLine("Account Number = " + m_objmfMicr.AccountNumber);
					sw.WriteLine("Amount         = " + m_objmfMicr.Amount);
					sw.WriteLine("Bank Number    = " + m_objmfMicr.BankNumber);
					sw.WriteLine("Serial Number  = " + m_objmfMicr.SerialNumber);
					sw.WriteLine("EPC Property   = " + m_objmfMicr.EPC);
					sw.WriteLine("Transit Number = " + m_objmfMicr.TransitNumber);
					sw.WriteLine("Check Type     = " + m_objmfMicr.CheckType.ToString());
					sw.WriteLine("Country Code   = " + m_objmfMicr.CountryCode.ToString());
				}
				if(m_objConfig.GetValueBool(Properties.OCR_AB))
				{
					int index = 0;
					int min = 100;
					int max = 0;
					int sum = 0;
					// OCR AB
					MFOcrAb mfOcrAb = new MFOcrAb();
					sw.WriteLine();
					sw.WriteLine();
					// Font designation
					if(m_objConfig.GetValue(Properties.OCR_AB_FONT) == 0)
					{
						sw.WriteLine("# OCR A character probabilities:");
						mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRA_ALPHANUM;
					}
					else
					{
						sw.WriteLine("# OCR B character probabilities:");
						mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRB_ALPHANUM;
					}
					mfOcrAb.StartX = OCR_AB_START_X;
					mfOcrAb.StartY = OCR_AB_START_Y;
					mfOcrAb.EndX = OCR_AB_END_X;
					mfOcrAb.EndY = OCR_AB_END_Y;
					mfOcrAb.Direction = MfDirection.MF_OCR_LEFTRIGHT;
					mfOcrAb.SpeceHandling = MfSpeceHandling.OCR_SPACE_ENABLE;

					// select front side image
					m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);

					// Acquire OCR_AB text
					m_errResultCode = m_objmfDevice.GetOcrABText(Convert.ToUInt32(transactionNumber), OcrImageSource.OCR_SOURCE_TRANSACTION_NUMBER, "", mfOcrAb);
					sw.WriteLine("");
					sw.WriteLine("OcrAbResult = " + m_errResultCode.ToString());

					// Acquire reliability percentage (maximum value/minimum value)
					for(index = 0; index < mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length - 1; index++)
					{
						sum = sum + (int)mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index);
						if(min > (int)mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index))
						{
							min = (int)mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index);
						}
						if(max < (int)mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index))
						{
							max = (int)mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index);
						}
					}
					if(index != 0)
					{
						double average = sum / mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length;

						sw.WriteLine();
						sw.WriteLine("OcrAbString = " + mfOcrAb.OcrReliableInfo.FirstSelectString);
						sw.WriteLine("NumberOfCharacters = " + mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length.ToString());
						sw.WriteLine("HighestPercentage  = " + max.ToString());
						sw.WriteLine("LowestPercentage   = " + min.ToString());
						sw.WriteLine("AveragePercentage  = " + average.ToString("G4"));
						sw.WriteLine();
						// Output the OCR_AB Text and reliability 
						for(index = 0; index <  mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length - 1; index++)
						{
							if(index < mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length)
							{
								sw.WriteLine("OcrPrimaryCharacter" + index.ToString() + "    = '" + mfOcrAb.OcrReliableInfo.FirstSelectString.ToString().Substring(index, 1) + "'");
								sw.WriteLine("OcrPrimaryPercentage" + index.ToString() + "   = " + mfOcrAb.OcrReliableInfo.FirstSelectPercentage.GetValue(index).ToString());
							}
							if(index < mfOcrAb.OcrReliableInfo.SecondSelectPercentage.Length)
							{
								sw.WriteLine("OcrSecondaryCharacter" + index.ToString() + "  = '" + mfOcrAb.OcrReliableInfo.SecondSelectString.ToString().Substring(index, 1) + "'");
								sw.WriteLine("OcrSecondaryPercentage" + index.ToString() + " = " + mfOcrAb.OcrReliableInfo.SecondSelectPercentage.GetValue(index).ToString());
							}
						}
					}
				}	
			}
			finally
			{
				sw.Close();				
			}
		}

		private void DisplayBarcodeData(int transactionNumber)
		{
			String strBarcodeData = "";
			if(m_objConfig.GetValueBool(Properties.BARCODE_FRONT_DETECT))
			{
				// select front side
				m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT);
				m_errResultCode = m_objmfDevice.GetBarcodeData(transactionNumber, m_objmfBarcodeFront);
				if(m_objmfBarcodeFront.Data != null) 
				{
					// convert string
					foreach(MFBarcode.BarcodeData objData in m_objmfBarcodeFront.Data)
					{
						char[] cData = new char[objData.Data.Length];
						for(int nCount = 0; nCount < cData.Length; nCount++)
						{
							cData[nCount] = (char)objData.Data[nCount];
						}
						strBarcodeData += new String(cData);
						strBarcodeData += " ";
					}
				}
				strBarcodeData = strBarcodeData.Replace("\0", " ");
				if(m_objmfBarcodeFront.Ret != ErrorCode.SUCCESS)
				{
					strBarcodeData += "(" + m_objmfBarcodeFront.Ret + ")";
				}
			}
			m_objTms1000.SetBarcodeFront(strBarcodeData);
			strBarcodeData = "";

			if(m_objConfig.GetValueBool(Properties.BARCODE_BACK_DETECT))
			{
				// select back side
				m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK);
				m_errResultCode = m_objmfDevice.GetBarcodeData(transactionNumber, m_objmfBarcodeBack);
				if(m_objmfBarcodeBack.Data != null) 
				{
					foreach(MFBarcode.BarcodeData objData in m_objmfBarcodeBack.Data)
					{
						// convert string
						char[] cData = new char[objData.Data.Length];
						for(int nCount = 0; nCount < cData.Length; nCount++)
						{
							cData[nCount] = (char)objData.Data[nCount];
						}
						strBarcodeData += new String(cData);
						strBarcodeData += " ";
					}
				}
				strBarcodeData = strBarcodeData.Replace("\0", " ");
				if(m_objmfBarcodeBack.Ret != ErrorCode.SUCCESS)
				{
					strBarcodeData += "(" + m_objmfBarcodeBack.Ret + ")";
				}
				m_objTms1000.SetBarcodeBack(strBarcodeData);
			}
		}

		// this method is return the  value of ErrorSelect 
		private MfErrorSelect GetDetect(bool bDetect)
		{
			if(bDetect)
			{
				return MfErrorSelect.MF_ERROR_SELECT_DETECT;
			}
			else
			{
				return MfErrorSelect.MF_ERROR_SELECT_NODETECT;
			}
		}

		// this method is return the  value of ErrorEject
		private MfEject GetEject(int iEject)
		{
			switch(iEject)
			{
				case 0:
					return MfEject.MF_EJECT_MAIN_POCKET;
				case 1:
					return MfEject.MF_EJECT_SUB_POCKET;
				case 2:
					return MfEject.MF_EJECT_NOEJECT;
				default:
					return MfEject.MF_EJECT_MAIN_POCKET;
			}
		}

		// this method is return the  value of Stamp
		private MfStamp GetStamp(bool bStamp)
		{
			if(bStamp)
			{
				return MfStamp.MF_STAMP_ENABLE;
			}
			else
			{
				return MfStamp.MF_STAMP_DISABLE;
			}
		}

		// this method is return the  value of Cancel
		private MfCancel GetCancel(bool bCancel)
		{
			if(bCancel)
			{
				return MfCancel.MF_CANCEL_ENABLE;
			}
			else
			{
				return MfCancel.MF_CANCEL_DISABLE;
			}
		}

		// Stop buzzer during ringing
		public void StopBuzzer()
		{
			CheckResponse(m_objmfDevice.RingBuzzer(0, 0, 0, 0), 
				"RingBuzzer(0, 0, 0, 0)");
		}

		// Acquire status
		public ASB GetStatus()
		{
			return m_objmfDevice.Status;
		}

		// Called when DeviceStatus status is sent
		public void Status(ASB Status)
		{
			// Set the status to DeviceStatus Form
			m_objTms1000.SetDeviceStatus(Status);
		}

		// Execute cleaning
		public void Cleaning()
		{
			CheckResponse(m_objmfDevice.MICRCleaning(), "MICRCleaning(Handle)");
		}

		// get error occured
		public string GetErrorOccured()
		{
			return m_errErrorOccured.ToString();
		}

		public TimeSpan GetStartWaitTime()
		{
			uint uiStartWaitTime = m_objmfProcess.StartWaitTime;
			if(uiStartWaitTime > 6400)
			{
				uiStartWaitTime = 6400;
			}
			return TimeSpan.FromMilliseconds((double)uiStartWaitTime);
		}
	}
}
