// APIUsage.cpp : implementation file
//

#include "stdafx.h"
#include "TM-S1000Sample.h"
#include "TM-S1000SampleDlg.h"
#include "APIUsage.h"

#define SCAN_SELECT_MULTIPLE	0
#define SCAN_SELECT_SINGLE		1

#define EJECT_SELECT_MAIN		0
#define EJECT_SELECT_SUB		1
#define EJECT_SELECT_NOEJECT	2

#define CONTINURE_SELECT_OVERLAP	0
#define CONTINURE_SELECT_NOOVERLAP	1
#define CONTINURE_SELECT_CANCEL		2

// This define is used primarily to check for errors and display an error dialog upon error
// in a release version program the error should be caught and handled depending on your business logic
#define CheckResponse(a) {int ret = a; if(ret != SUCCESS) {::MessageBox(NULL,#a, GetResultString(ret),MB_OK | MB_ICONERROR); return;}}

// CAPIUsage

// This is the callback method that is called when the scan status changes
// The information is forwarded to our single CAPIUsage class
int CALLBACK cbScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	CTMS1000SampleDlg* pDlg = (CTMS1000SampleDlg*)(theApp.m_pMainWnd);
	if(pDlg != NULL){
		pDlg->m_api.ScanStatus(dwTransactionNumber, wMainStatus, wSubStatus, pPortName);
	}
	return 0;
}

IMPLEMENT_DYNAMIC(CAPIUsage, CWnd)
CAPIUsage::CAPIUsage()
{
	// Initialize Internal member parameters
	m_pProperties = &(theApp.m_Properties);
	m_hScanEvent = ::CreateEvent(NULL, TRUE, FALSE, NULL);
	m_hCancelHandle = ::CreateEvent(NULL, TRUE, FALSE, NULL);
	m_iHandle = 0;
	m_wErrorOccured = SUCCESS;

	long lLoadLibrary = InitInstance();
	if(lLoadLibrary != SUCCESS){
		::MessageBox(NULL, _T("Failure LoadLibrary"), GetResultString(lLoadLibrary),MB_OK);
		return;
	}

	// The first thing to do with the status API is open a connection to the printer
	BiESCNEnable(CROP_STORE_FILE);
	// Connect to the printer
	m_iHandle = BiOpenMonPrinter(TYPE_PRINTER, "TM-S1000U");
	if(m_iHandle < SUCCESS){
		::MessageBox(NULL,_T("Unable to connect to printer\n\nEnsure Driver installed correctly\nand Printer is powered on"),GetResultString(m_iHandle),MB_OK);
		return;
	}

	// Once we have a printer connection, Callback handlers are registered
	// it is also possible to set and cancel callbacks so they are fired only when you wish
	// Register the callback destination
	CheckResponse(BiSCNMICRSetStatusBackFunction(m_iHandle, cbScanStatus));
}


BEGIN_MESSAGE_MAP(CAPIUsage, CWnd)
END_MESSAGE_MAP()


CAPIUsage::~CAPIUsage()
{
	if(m_hLibrary != NULL){
		CancelScan();
		End();
		::FreeLibrary(m_hLibrary);
	}
	CloseHandle(m_hScanEvent);
	CloseHandle(m_hCancelHandle);
}

void CAPIUsage::CancelScan()
{
	SetEvent(m_hCancelHandle);
	CheckResponse(BiSCNMICRCancelFunction(m_iHandle, MF_EJECT_DISCHARGE));
}

void CAPIUsage::End()
{
	SetEvent(m_hCancelHandle);
	if(m_iHandle > SUCCESS){
		BiSCNMICRCancelFunction(m_iHandle, MF_EJECT_DISCHARGE);
		BiSCNMICRCancelStatusBack(m_iHandle);
		BiCloseMonPrinter(m_iHandle);
		m_iHandle = 0;
	}
}

// CAPIUsage message handlers

long CAPIUsage::InitInstance()
{
	m_hLibrary = ::LoadLibrary("EpsStmApiWrapper.dll");
	long lRet = ::GetLastError();

	if(lRet != SUCCESS){
		return lRet;
	}

	BiOpenMonPrinter = (DLL_BiOpenMonPrinter)GetProcAddress(m_hLibrary, "BiOpenMonPrinter");
	BiSetMonInterval = (DLL_BiSetMonInterval)GetProcAddress(m_hLibrary, "BiSetMonInterval");
	BiGetStatus = (DLL_BiGetStatus)GetProcAddress(m_hLibrary, "BiGetStatus");
	BiSetStatusBackFunction = (DLL_BiSetStatusBackFunction)GetProcAddress(m_hLibrary, "BiSetStatusBackFunction");
	BiSetStatusBackWnd = (DLL_BiSetStatusBackWnd)GetProcAddress(m_hLibrary, "BiSetStatusBackWnd");
	BiCancelStatusBack = (DLL_BiCancelStatusBack)GetProcAddress(m_hLibrary, "BiCancelStatusBack");
	BiResetPrinter = (DLL_BiResetPrinter)GetProcAddress(m_hLibrary, "BiResetPrinter");
	BiGetCounter = (DLL_BiGetCounter)GetProcAddress(m_hLibrary, "BiGetCounter");
	BiResetCounter = (DLL_BiResetCounter)GetProcAddress(m_hLibrary, "BiResetCounter");
	BiCancelError = (DLL_BiCancelError)GetProcAddress(m_hLibrary, "BiCancelError");
	BiGetType = (DLL_BiGetType)GetProcAddress(m_hLibrary, "BiGetType");
	BiGetOfflineCode = (DLL_BiGetOfflineCode)GetProcAddress(m_hLibrary, "BiGetOfflineCode");
	BiGetOfflineCodeByIndex = (DLL_BiGetOfflineCodeByIndex)GetProcAddress(m_hLibrary, "BiGetOfflineCodeByIndex");
	BiMICRSelectDataHandling = (DLL_BiMICRSelectDataHandling)GetProcAddress(m_hLibrary, "BiMICRSelectDataHandling");
	BiMICRGetStatus = (DLL_BiMICRGetStatus)GetProcAddress(m_hLibrary, "BiMICRGetStatus");
	BiMICRCleaning = (DLL_BiMICRCleaning)GetProcAddress(m_hLibrary, "BiMICRCleaning");
	BiSCNSetImageQuality = (DLL_BiSCNSetImageQuality)GetProcAddress(m_hLibrary, "BiSCNSetImageQuality");
	BiSCNSetImageFormat = (DLL_BiSCNSetImageFormat)GetProcAddress(m_hLibrary, "BiSCNSetImageFormat");
	BiSCNSetScanArea = (DLL_BiSCNSetScanArea)GetProcAddress(m_hLibrary, "BiSCNSetScanArea");
	BiSCNGetImageQuality = (DLL_BiSCNGetImageQuality)GetProcAddress(m_hLibrary, "BiSCNGetImageQuality");
	BiSCNGetImageFormat = (DLL_BiSCNGetImageFormat)GetProcAddress(m_hLibrary, "BiSCNGetImageFormat");
	BiSCNGetScanArea = (DLL_BiSCNGetScanArea)GetProcAddress(m_hLibrary, "BiSCNGetScanArea");
	BiSCNSetCroppingArea = (DLL_BiSCNSetCroppingArea)GetProcAddress(m_hLibrary, "BiSCNSetCroppingArea");
	BiSCNGetCroppingArea = (DLL_BiSCNGetCroppingArea)GetProcAddress(m_hLibrary, "BiSCNGetCroppingArea");
	BiSCNDeleteCroppingArea = (DLL_BiSCNDeleteCroppingArea)GetProcAddress(m_hLibrary, "BiSCNDeleteCroppingArea");
	BiSCNSelectScanUnit = (DLL_BiSCNSelectScanUnit)GetProcAddress(m_hLibrary, "BiSCNSelectScanUnit");
	BiSCNMICRFunction = (DLL_BiSCNMICRFunction)GetProcAddress(m_hLibrary, "BiSCNMICRFunction");
	BiSCNMICRCancelFunction = (DLL_BiSCNMICRCancelFunction)GetProcAddress(m_hLibrary, "BiSCNMICRCancelFunction");
	BiSCNSelectScanFace = (DLL_BiSCNSelectScanFace)GetProcAddress(m_hLibrary, "BiSCNSelectScanFace");
	BiGetPrnCapability = (DLL_BiGetPrnCapability)GetProcAddress(m_hLibrary, "BiGetPrnCapability");
	BiCloseMonPrinter = (DLL_BiCloseMonPrinter)GetProcAddress(m_hLibrary, "BiCloseMonPrinter");
	BiSetStatusBackFunctionEx = (DLL_BiSetStatusBackFunctionEx)GetProcAddress(m_hLibrary, "BiSetStatusBackFunctionEx");
	BiSCNMICRFunctionContinuously = (DLL_BiSCNMICRFunctionContinuously)GetProcAddress(m_hLibrary, "BiSCNMICRFunctionContinuously");
	BiSCNMICRFunctionPostPrint = (DLL_BiSCNMICRFunctionPostPrint)GetProcAddress(m_hLibrary, "BiSCNMICRFunctionPostPrint");
	BiSCNMICRSetStatusBackFunction = (DLL_BiSCNMICRSetStatusBackFunction)GetProcAddress(m_hLibrary, "BiSCNMICRSetStatusBackFunction");
	BiSCNMICRSetStatusBackWnd = (DLL_BiSCNMICRSetStatusBackWnd)GetProcAddress(m_hLibrary, "BiSCNMICRSetStatusBackWnd");
	BiSCNMICRCancelStatusBack = (DLL_BiSCNMICRCancelStatusBack)GetProcAddress(m_hLibrary, "BiSCNMICRCancelStatusBack");
	BiGetMicrText = (DLL_BiGetMicrText)GetProcAddress(m_hLibrary, "BiGetMicrText");
	BiGetScanImage = (DLL_BiGetScanImage)GetProcAddress(m_hLibrary, "BiGetScanImage");
	BiGetTransactionNumber = (DLL_BiGetTransactionNumber)GetProcAddress(m_hLibrary, "BiGetTransactionNumber");
	BiSetTransactionNumber = (DLL_BiSetTransactionNumber)GetProcAddress(m_hLibrary, "BiSetTransactionNumber");
	BiGetPrintStation = (DLL_BiGetPrintStation)GetProcAddress(m_hLibrary, "BiGetPrintStation");
	BiSetPrintStation = (DLL_BiSetPrintStation)GetProcAddress(m_hLibrary, "BiSetPrintStation");
	BiPrintText = (DLL_BiPrintText)GetProcAddress(m_hLibrary, "BiPrintText");
	BiPrintImage = (DLL_BiPrintImage)GetProcAddress(m_hLibrary, "BiPrintImage");
	BiGetPrintSize = (DLL_BiGetPrintSize)GetProcAddress(m_hLibrary, "BiGetPrintSize");
	BiSetPrintSize = (DLL_BiSetPrintSize)GetProcAddress(m_hLibrary, "BiSetPrintSize");
	BiGetPrintPosition = (DLL_BiGetPrintPosition)GetProcAddress(m_hLibrary, "BiGetPrintPosition");
	BiSetPrintPosition = (DLL_BiSetPrintPosition)GetProcAddress(m_hLibrary, "BiSetPrintPosition");
	BiBufferedPrint = (DLL_BiBufferedPrint)GetProcAddress(m_hLibrary, "BiBufferedPrint");
	BiSetTransactionNumberWithIncremental = (DLL_BiSetTransactionNumberWithIncremental)GetProcAddress(m_hLibrary, "BiSetTransactionNumberWithIncremental");
	BiSetBehaviorToScnResult = (DLL_BiSetBehaviorToScnResult)GetProcAddress(m_hLibrary, "BiSetBehaviorToScnResult");
	BiRingBuzzer = (DLL_BiRingBuzzer)GetProcAddress(m_hLibrary, "BiRingBuzzer");

	BiESCNEnable = (DLL_BiESCNEnable)GetProcAddress(m_hLibrary, "BiESCNEnable");
	BiESCNGetAutoSize = (DLL_BiESCNGetAutoSize)GetProcAddress(m_hLibrary, "BiESCNGetAutoSize");
	BiESCNSetAutoSize = (DLL_BiESCNSetAutoSize)GetProcAddress(m_hLibrary, "BiESCNSetAutoSize");
	BiESCNGetCutSize = (DLL_BiESCNGetCutSize)GetProcAddress(m_hLibrary, "BiESCNGetCutSize");
	BiESCNSetCutSize = (DLL_BiESCNSetCutSize)GetProcAddress(m_hLibrary, "BiESCNSetCutSize");
	BiESCNGetRotate = (DLL_BiESCNGetRotate)GetProcAddress(m_hLibrary, "BiESCNGetRotate");
	BiESCNSetRotate = (DLL_BiESCNSetRotate)GetProcAddress(m_hLibrary, "BiESCNSetRotate");
	BiESCNGetDocumentSize = (DLL_BiESCNGetDocumentSize)GetProcAddress(m_hLibrary, "BiESCNGetDocumentSize");
	BiESCNSetDocumentSize = (DLL_BiESCNSetDocumentSize)GetProcAddress(m_hLibrary, "BiESCNSetDocumentSize");
	BiESCNDefineCropArea = (DLL_BiESCNDefineCropArea)GetProcAddress(m_hLibrary, "BiESCNDefineCropArea");
	BiESCNGetMaxCropAreas = (DLL_BiESCNGetMaxCropAreas)GetProcAddress(m_hLibrary, "BiESCNGetMaxCropAreas");
	BiESCNStoreImage = (DLL_BiESCNStoreImage)GetProcAddress(m_hLibrary, "BiESCNStoreImage");
	BiESCNRetrieveImage = (DLL_BiESCNRetrieveImage)GetProcAddress(m_hLibrary, "BiESCNRetrieveImage");
	BiESCNClearImage = (DLL_BiESCNClearImage)GetProcAddress(m_hLibrary, "BiESCNClearImage");
	BiESCNGetRemainingImages = (DLL_BiESCNGetRemainingImages)GetProcAddress(m_hLibrary, "BiESCNGetRemainingImages");

	return lRet;
}

// Return any of the designated handles that signal
// Return TRUE if the handle passed to the parameter signals
// Return FALSE if m_hCancelHandle signals
BOOL CAPIUsage::MsgWaitForMultipleObjectsInfinite(HANDLE hEventHandle)
{
	HANDLE hHandle[2] = {hEventHandle, m_hCancelHandle};
	while(TRUE){
		switch(MsgWaitForMultipleObjects(2, hHandle, FALSE, INFINITE, QS_ALLINPUT)){
			case (WAIT_OBJECT_0 + 2):
				// If a message comes, and all the handles are not signaling, 
				// transmit the message
				MSG msg; 
				while (PeekMessage(&msg, NULL, 0, 0, PM_REMOVE))
					DispatchMessage(&msg); 
				break;
			case WAIT_OBJECT_0:
				return TRUE;	// hHandle[0] signals
			default:
				return FALSE;	// hHandle[1] signals
		}
	}
}

// This method is called when the program starts or the user changes configuration
// operation is offloaded to the correct ConfigureXxx function
void CAPIUsage::Configure()
{
	// DPI setting method
	// Specify MF_SCAN_DPI_XXX in sResolution of the MF_SCAN structure
	// Execute BiSCNMICRFunctionXXX
	// * Execute before scanning

	// Sharpening setting method
	// Specify EPS_BI_SCN_XXX as the 5th parameter of BiSCNSetImageQuality and 	execute it (other parameters are discretionary)
	// * Execute before scanning

	// Setting method for multiple electronic endorsements
	// Execute MF_PRT_BUFFERING with BiBufferedPrint
	// Set multiple electronic endorsements
	// Execute MF_PRT_EXEC with BiBufferedPrint
	// * Execute before scanning

	// Set station electronic endorse
	CheckResponse(BiSetPrintStation(m_iHandle, MF_ST_E_ENDORSEMENT));

	// Delete the registered electronic endorsement
	CheckResponse(BiBufferedPrint(m_iHandle, MF_PRT_CLEAR));

	switch(m_pProperties->GetValue(SCAN_FUNC)){
		case SCAN_SELECT_MULTIPLE:
			ConfigureMultiple();
			break;
		case SCAN_SELECT_SINGLE:
			ConfigureSingle();
			break;
		default:
			ConfigureMultiple();
			break;
	}
}

// this method is return MF_EJECT_XXX
BYTE CAPIUsage::ChangeEject(int iEject)
{
	BYTE bEject = 0;
	switch(iEject){
		case EJECT_SELECT_MAIN:
			bEject =  MF_EJECT_MAIN_POCKET;
			break;
		case EJECT_SELECT_SUB:
			bEject =  MF_EJECT_SUB_POCKET;
			break;
		case EJECT_SELECT_NOEJECT:
			bEject =  MF_EJECT_NOEJECT;
			break;
		default:
			bEject =  MF_EJECT_MAIN_POCKET;
			break;
	}

	return bEject;
}

// this method is return MF_PROCESS_CONTINUE_XXXXX
BYTE CAPIUsage::ChangeNextCheck(int iNextCheck)
{
	BYTE bNextCheck = 0;
	switch(iNextCheck){
		case CONTINURE_SELECT_OVERLAP:
			bNextCheck =  MF_PROCESS_CONTINUE_OVERLAP;
			break;
		case CONTINURE_SELECT_NOOVERLAP:
			bNextCheck =  MF_PROCESS_CONTINUE_NOOVERLAP;
			break;
		case CONTINURE_SELECT_CANCEL:
			bNextCheck =  MF_PROCESS_CONTINUE_CANCEL;
			break;
		default:
			bNextCheck =  MF_PROCESS_CONTINUE_OVERLAP;
			break;
	}

	return bNextCheck;
}

// This method is endorse setting
void CAPIUsage::ConfigureEndorseText()
{
	// set position
	CheckResponse(BiSetPrintPosition(m_iHandle, 10, 60));

	// set endorse string
	MF_DECORATE mfDecorate;
	mfDecorate.dwAttribute = MF_PRINT_BLACK;
	mfDecorate.wFont = MF_PRINT_SYSTEMFONT;
	mfDecorate.szFontName = "Arial";
	mfDecorate.wFontSize = 30;
	CheckResponse(BiPrintText(m_iHandle, "Transaction#:<00000000>\x0D\x0A", mfDecorate));
}

// Set image endorsement
void CAPIUsage::ConfigureEndorseImage()
{
	// set position
	CheckResponse(BiSetPrintPosition(m_iHandle, 100, 30));

	// set size
	CheckResponse(BiSetPrintSize(m_iHandle, 30, 30));

	// set endowse image
	CheckResponse(BiPrintImage(m_iHandle, "Image.jpg"));
}

// This method is Process struct setting
void CAPIUsage::SetProcessStruct()
{
	// ActivationMode
	m_tProcess.bActivationMode = (BYTE)m_pProperties->GetValueDefTrue(ACTIVATE_MODE);

	// PaperMisInsertion
	m_tProcess.bPaperMisInsertionErrorSelect = (BYTE)m_pProperties->GetValueDefTrue(PAPER_MIS_INSERT_DETECT);
	m_tProcess.bPaperMisInsertionErrorEject = ChangeEject(m_pProperties->GetValueDefFalse(PAPER_MIS_INSERT_EJECT));
	m_tProcess.bPaperMisInsertionStamp = (BYTE)m_pProperties->GetValueDefFalse(PAPER_MIS_INSERT_STAMP);
	m_tProcess.bPaperMisInsertionCancel = (BYTE)m_pProperties->GetValueDefFalse(PAPER_MIS_INSERT_CANCEL);

	// Noise
	m_tProcess.bNoiseErrorSelect = (BYTE)m_pProperties->GetValueDefTrue(NOISE_DETECT);
	m_tProcess.bNoiseErrorEject = ChangeEject(m_pProperties->GetValueDefFalse(NOISE_EJECT));
	m_tProcess.bNoiseStamp = (BYTE)m_pProperties->GetValueDefFalse(NOISE_STAMP);
	m_tProcess.bNoiseCancel = (BYTE)m_pProperties->GetValueDefTrue(NOISE_CANCEL);

	// DoubleFeed
	m_tProcess.bDoubleFeedErrorSelect = (BYTE)m_pProperties->GetValueDefTrue(DOUBLE_FEED_DETECT);
	m_tProcess.bDoubleFeedErrorEject = ChangeEject(m_pProperties->GetValueDefFalse(DOUBLE_FEED_EJECT));
	m_tProcess.bDoubleFeedStamp = (BYTE)m_pProperties->GetValueDefFalse(DOUBLE_FEED_STAMP);
	m_tProcess.bDoubleFeedCancel = (BYTE)m_pProperties->GetValueDefFalse(DOUBLE_FEED_CANCEL);

	// Baddata
	m_tProcess.bBaddataCount = (BYTE)m_pProperties->GetValue(BADDATA_COUNT);
	m_tProcess.bBaddataErrorSelect = (BYTE)m_pProperties->GetValueDefTrue(BADDATA_DETECT);
	m_tProcess.bBaddataErrorEject = ChangeEject(m_pProperties->GetValueDefFalse(BADDATA_EJECT));
	m_tProcess.bBaddataStamp = (BYTE)m_pProperties->GetValueDefFalse(BADDATA_STAMP);
	m_tProcess.bBaddataCancel = (BYTE)m_pProperties->GetValueDefFalse(BADDATA_CANCEL);

	// Nodata
	m_tProcess.bNodataErrorSelect = (BYTE)m_pProperties->GetValueDefTrue(NODATA_DETECT);
	m_tProcess.bNodataErrorEject = ChangeEject(m_pProperties->GetValueDefFalse(NODATA_EJECT));
	m_tProcess.bNodataStamp = (BYTE)m_pProperties->GetValueDefFalse(NODATA_STAMP);
	m_tProcess.bNodataCancel = (BYTE)m_pProperties->GetValueDefFalse(NODATA_CANCEL);

}

// this method configures the parameters used while scanning single checks
// primarily the BiSCNMICRFunctionContinuously method is used
void CAPIUsage::ConfigureMultiple()
{
	// Base
	m_tBase01.iSize = sizeof(MF_BASE01);
	m_tBase01.iVersion = MF_BASE_VERSION01;
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tBase01, MF_GET_BASE_DEFAULT));
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tBase01, MF_SET_BASE_PARAM));

	// Scan front
	m_tScanFront.iSize = sizeof(MF_SCAN);
	m_tScanFront.iVersion = MF_SCAN_VERSION;
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tScanFront, MF_GET_SCAN_DEFAULT));
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tScanFront, MF_SET_SCAN_FRONT_PARAM));

	// Scan back
	m_tScanBack.iSize = sizeof(MF_SCAN);
	m_tScanBack.iVersion = MF_SCAN_VERSION;
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tScanBack, MF_GET_SCAN_DEFAULT));
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tScanBack, MF_SET_SCAN_BACK_PARAM));

	// Micr
	m_tMicr.iSize = sizeof(MF_MICR);
	m_tMicr.iVersion = MF_MICR_VERSION;
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tMicr, MF_GET_MICR_DEFAULT));
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tMicr, MF_SET_MICR_PARAM));

	// Print
	m_tPrint01.iSize = sizeof(MF_PRINT01);
	m_tPrint01.iVersion = MF_PRINT_VERSION01;
	// Set the MF_PRINT_TYPE_ELECTRIC_ENDORSE_EXTEND to m_tPrint01.dwEndorseType
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tPrint01, MF_GET_PRINT_DEFAULT));
	// An error when the lpString array is all NULL
	m_tPrint01.lpString[0] = "";	// Specify blank characters
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tPrint01, MF_SET_PRINT_PARAM));

	// Operational settings when an error occurs
	// Process
	m_tProcess.iSize = sizeof(MF_PROCESS);
	m_tProcess.iVersion = MF_PROCESS_VERSION;
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tProcess, MF_GET_PROCESS_DEFAULT));
	SetProcessStruct();
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, &m_tProcess, MF_SET_PROCESS_PARAM));

	// Set the endorsement
	if(m_pProperties->GetValueDefFalse(ELEC_ENDORSE_IMAGE)){
		// Set the electronic endorsement
		CheckResponse(BiBufferedPrint(m_iHandle, MF_PRT_BUFFERING));

		// Set the image electronic endorsement
		ConfigureEndorseImage();

		// Register the electronic endorsement set
		CheckResponse(BiBufferedPrint(m_iHandle, MF_PRT_EXEC));
	}
}

// this method configures the parameters used while scanning single checks
// primarily the BiSCNMICRFunctionPostPrint method is used
void CAPIUsage::ConfigureSingle()
{
	// Base
	m_tBase01.iSize = sizeof(MF_BASE01);
	m_tBase01.iVersion = MF_BASE_VERSION01;
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tBase01, MF_GET_BASE_DEFAULT));
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tBase01, MF_SET_BASE_PARAM));

	// Scan front
	m_tScanFront.iSize = sizeof(MF_SCAN);
	m_tScanFront.iVersion = MF_SCAN_VERSION;
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tScanFront, MF_GET_SCAN_DEFAULT));
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tScanFront, MF_SET_SCAN_FRONT_PARAM));

	// Scan back
	m_tScanBack.iSize = sizeof(MF_SCAN);
	m_tScanBack.iVersion = MF_SCAN_VERSION;
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tScanBack, MF_GET_SCAN_DEFAULT));
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tScanBack, MF_SET_SCAN_BACK_PARAM));

	// Micr
	m_tMicr.iSize = sizeof(MF_MICR);
	m_tMicr.iVersion = MF_MICR_VERSION;
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tMicr, MF_GET_MICR_DEFAULT));
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tMicr, MF_SET_MICR_PARAM));

	// Print
	m_tPrint01.iSize = sizeof(MF_PRINT01);
	m_tPrint01.iVersion = MF_PRINT_VERSION01;
	// Set the MF_PRINT_TYPE_ELECTRIC_ENDORSE_EXTEND to m_tPrint01.dwEndorseType
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tPrint01, MF_GET_PRINT_DEFAULT));
	// An error when the lpString array is all NULL
	m_tPrint01.lpString[0] = "";	// Specify blank characters
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tPrint01, MF_SET_PRINT_PARAM));

	// Operational settings when an error occurs
	// Process
	m_tProcess.iSize = sizeof(MF_PROCESS);
	m_tProcess.iVersion = MF_PROCESS_VERSION;
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tProcess, MF_GET_PROCESS_DEFAULT));
	SetProcessStruct();
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, &m_tProcess, MF_SET_PROCESS_PARAM));

	// Set the endorsement
	if(m_pProperties->GetValueDefFalse(ELEC_ENDORSE_IMAGE)){
		// Set the electronic endorsement
		CheckResponse(BiBufferedPrint(m_iHandle, MF_PRT_BUFFERING));

		// Set the image electronic endorsement
		ConfigureEndorseImage();

		// Register the electronic endorsement set
		CheckResponse(BiBufferedPrint(m_iHandle, MF_PRT_EXEC));
	}
}

// this method is return transaction number
DWORD CAPIUsage::GetTransactonNumber()
{
	DWORD dwTransactionNumber = 0;
	BiGetTransactionNumber(m_iHandle, &dwTransactionNumber);
	return dwTransactionNumber;
}

// This method is called when the user clicks the scan button
// operation is offloaded to the correct ScanXxx function
void CAPIUsage::Scan()
{
	ResetEvent(m_hCancelHandle);
	ResetEvent(m_hScanEvent);
	switch(m_pProperties->GetValue(SCAN_FUNC)){
		case SCAN_SELECT_MULTIPLE:
			ScanMultiple();
			break;
		case SCAN_SELECT_SINGLE:
			ScanSingle();
			break;
		default:
			ScanMultiple();
	}
}

// this method performs scanning of a multi check
// BiSCNMICRFunctionContinuously(MF_EXEC) is called
void CAPIUsage::ScanMultiple()
{
	CheckResponse(BiSCNMICRFunctionContinuously(m_iHandle, NULL, MF_EXEC));

	MsgWaitForMultipleObjectsInfinite(m_hScanEvent);
}

// this method performs scanning of a single check
// BiSCNMICRFunctionPostPrint(MF_EXEC) is called
void CAPIUsage::ScanSingle()
{
	CheckResponse(BiSCNMICRFunctionPostPrint(m_iHandle, NULL, MF_EXEC));

	MsgWaitForMultipleObjectsInfinite(m_hScanEvent);
}

// This method is called whenever the scan state changes
// Called when scanning with BiSCNMICRFunctionPostPrint/BiSCNMICRFunctionContinuously
void CAPIUsage::ScanStatus(DWORD dwTransactionNumber, WORD wMainStatus, WORD wSubStatus, LPSTR pPortName)
{
	// There are many varying status changes that you are notified of via this method.
	// 2 are very important
	// MF_DATARECEIVE_DONE - notification that a check is finished scanning
	// MF_FUNCTION_DONE - all checks have finished scanning
	// you may wish to add handling for other status types to meet business requirements

	int iResult = 0;
	CTMS1000SampleDlg* pDlg = (CTMS1000SampleDlg*)theApp.m_pMainWnd;

	if(wMainStatus == MF_ERROR_OCCURED){
		m_wErrorOccured = wSubStatus;
	}

	// Data receive complete
	if(wMainStatus == MF_DATARECEIVE_DONE){

		// null check
		if(pDlg != NULL){

			CString strMicr;

			// get micr data
			::ZeroMemory(m_tMicr.szMicrStr, sizeof(m_tMicr.szMicrStr));
			// Specify the relevant item for MICR text
			m_tMicr.bMicOcrSelect = MF_MICR_USE_MICR;
			// Acquire MICR text
			iResult = BiGetMicrText(m_iHandle, dwTransactionNumber, &m_tMicr);
			// Copy the text acquired
			strMicr = m_tMicr.szMicrStr;
			if(iResult != SUCCESS){
				// Add error code
				strMicr += " ( ";
				strMicr += GetResultString(iResult);
				strMicr += " ) ";
			}
			pDlg->SetMicrString(strMicr);

			// get front image
			// Set the relevant item for the front side image
			iResult = BiSCNSelectScanFace(m_iHandle, MF_SCAN_FACE_FRONT);

			if(iResult == SUCCESS){
				// Specify the file format in BMP
				iResult = BiSCNSetImageFormat(m_iHandle, EPS_BI_SCN_BITMAP);

				if(iResult == SUCCESS){
					// Acquire image data
					iResult = BiGetScanImage(m_iHandle, dwTransactionNumber, &m_tScanFront);

					if(iResult == SUCCESS && m_tScanFront.lpbScanData != NULL){
						// Display
						pDlg->SetImage(m_tScanFront.lpbScanData, m_tScanFront.dwScanSize, TRUE);
						GlobalFree(m_tScanFront.lpbScanData);
						m_tScanFront.lpbScanData = NULL;
					}
				}
			}

			// set electronic endorse
			if(m_pProperties->GetValueDefFalse(ELEC_ENDORSE_TEXT)){
				// endorse setting
				ConfigureEndorseText();
			}

			// get back image
			// Set the relevant item for the front side
			iResult = BiSCNSelectScanFace(m_iHandle, MF_SCAN_FACE_BACK);

			if(iResult == SUCCESS){
				// Specify the file format in BMP
				iResult = BiSCNSetImageFormat(m_iHandle, EPS_BI_SCN_BITMAP);

				if(iResult == SUCCESS){
					// Acquire image data
					iResult = BiGetScanImage(m_iHandle, dwTransactionNumber, &m_tScanBack);

					if(iResult == SUCCESS && m_tScanBack.lpbScanData != NULL){
						// Display
						pDlg->SetImage(m_tScanBack.lpbScanData, m_tScanBack.dwScanSize, FALSE);
						GlobalFree(m_tScanBack.lpbScanData);
						m_tScanBack.lpbScanData = NULL;
					}
				}
			}

			// Is it Confirmation Mode?
			if(!m_pProperties->GetValueDefTrue(ACTIVATE_MODE)){
				// Execute BiSetBehaviorToScnResult?
				if(m_pProperties->GetValueDefFalse(RUN_TO_SCN_RESULT)){
					// Call BiSetBehaviorToScnResult?
					if(!m_pProperties->GetValueDefFalse(NO_CALL_TO_SCN_RESULT)){
						// Display BiSetBehaviorToScnResult dialog box
						// Do no execute if it is canceled in the dialog box
						if(pDlg->SetConfirmationMode()){
							
							BiSetBehaviorToScnResult(	m_iHandle,
														ChangeEject(m_pProperties->GetValue(CONFIRMATION_EJECT)), 
														(BYTE)m_pProperties->GetValueDefFalse(CONFIRMATION_STAMP),
														ChangeNextCheck(m_pProperties->GetValue(CONFIRMATION_NEXT_CHECK)));
						}
					}else{
						// Use the value set without calling BiSetBehaviorToScnResult
						BiSetBehaviorToScnResult(	m_iHandle,
													ChangeEject(m_pProperties->GetValue(CONFIRMATION_EJECT)), 
													(BYTE)m_pProperties->GetValueDefFalse(CONFIRMATION_STAMP),
													ChangeNextCheck(m_pProperties->GetValue(CONFIRMATION_NEXT_CHECK)));
					}
				}
			}
			m_wErrorOccured = SUCCESS;
		}
	}

	// when all scanning complete notify scan method through the use of an event
	// once event is detected the scan method will return
	// Finish scan process
	if(wMainStatus == MF_FUNCTION_DONE){
		::SetEvent(m_hScanEvent);
	}
}

// Get ErrorOccured
LPCTSTR CAPIUsage::GetErrorOccured()
{
	return GetResultString((short)m_wErrorOccured);
}
DWORD CAPIUsage::GetStartWaitTime()
{
	DWORD dwStartWaitTime = m_tProcess.dwStartWaitTime;
	if(dwStartWaitTime > 6400) {
		dwStartWaitTime = 6400;
	}
	return dwStartWaitTime;
}
