Imports com.epson.bank.driver


Public Class ApiUsage
    Protected m_objMainForm As MainForm = Nothing
    WithEvents m_objmfDevice As MFDevice = Nothing
    Dim m_objmfBase As MFBase = Nothing
    Dim m_objmfMicr As MFMicr = Nothing
    Dim m_objmfScanFront As MFScan = Nothing
    Dim m_objmfScanBack As MFScan = Nothing
    Dim m_objmfPrint As MFPrint = Nothing
    Dim m_objmfProcess As MFProcess = Nothing
    Dim m_errResultCode As ErrorCode = ErrorCode.SUCCESS

    Public Sub New()
        ' initialize members
        MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY)
        ' Connect to the printer
        m_objmfDevice = New MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U")
        m_objmfBase = New MFBase
        m_objmfMicr = New MFMicr
        m_objmfScanFront = New MFScan
        m_objmfScanBack = New MFScan
        m_objmfPrint = New MFPrint
        m_objmfProcess = New MFProcess

        If (CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer" & ControlChars.NewLine & ControlChars.NewLine & _
                                "Ensure Driver installed correctly" & ControlChars.NewLine & "and Printer is powered on")) Then
            Exit Sub
        End If

        ' register for events
        ' Register the callback destination
        m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack()

        If (CheckResponse(m_errResultCode, "SCNMICRSetStatusBack")) Then
        End If
    End Sub

    ' this method is called when the user click the exit button
    Public Sub Close()
        If Not (m_objmfDevice Is Nothing) Then
            m_objmfDevice.SCNMICRCancelStatusBack()
            m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE)
            m_objmfDevice.CloseMonPrinter()
        End If
    End Sub

    ' this method is called when the user click the cancel button
    Public Sub CancelScan()
        If Not (m_objmfDevice Is Nothing) Then
            Call CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE), _
                                "SCNMICRCancelFunction(MF_EJECT_DISCHARGE)")
        End If
    End Sub

    ' copy MianForm object
    Public Sub Configure()
        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionPostPrint(mfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionPostPrint(mfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                            "SCNMICRFunctionPostPrint(mfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
            Return
        End If

        ' ScanBack
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                            "SCNMICRFunctionPostPrint(mfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
            Return
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionPostPrint(mfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionPostPrint(mfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionPostPrint(mfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

    End Sub

    ' copy MianForm object
    Public Sub MainFormItem(ByVal Item As mainForm)
        m_objMainForm = Item
    End Sub

    ' this method is called when the user click the scan button
    Public Sub Scan()
        m_errResultCode = m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC)

        If (CheckResponse(m_errResultCode, "SCNMICRFunctionPostPrint(MF_EXEC)")) Then
            Complete(0)
        End If
    End Sub

    ' This method is called when the Complete scan
    Private Sub Complete(ByVal transactionNumber As Integer)
        If (m_objMainForm Is Nothing) Then
            Return
        End If
        m_objMainForm.ScanComplete(0)
    End Sub

    ' This method is called whenever the scan state changes
    Private Sub SCNMICRSetStatusBack(ByVal transactionNumber As Integer, ByVal mainStatus As MainStatus, ByVal subStatus As ErrorCode, ByVal portName As String) Handles m_objmfDevice.SCNMICRStatusCallback
        ' Finish scan process
        If (mainStatus = mainStatus.MF_FUNCTION_DONE) Then
            Complete(0)
        End If
    End Sub

    ' this method is return response
    ' SUCCESS : Return to FALSE
    ' ERROR : Display the message box, return TRUE
    Private Function CheckResponse(ByVal errCode As ErrorCode, Optional ByVal str As String = "Error") As Boolean
        If Not (ErrorCode.SUCCESS.Equals(errCode)) Then
            MessageBox.Show(str, errCode.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
        End If
        Return False
    End Function
End Class
