Imports com.epson.bank.driver


Public Class ApiUsage
    Protected m_objMainForm As MainForm = Nothing
    Dim WithEvents m_objmfDevice As MFDevice = Nothing
    Dim m_objmfBase As MFBase = Nothing
    Dim m_objmfMicr As MFMicr = Nothing
    Dim m_objmfScanFront As MFScan = Nothing
    Dim m_objmfScanBack As MFScan = Nothing
    Dim m_objmfPrint As MFPrint = Nothing
    Dim m_objmfProcess As MFProcess = Nothing
    Dim m_errResultCode As ErrorCode = ErrorCode.SUCCESS
    Dim m_IAar As System.IAsyncResult = Nothing
    Dim m_UpdateCallback As UpdateCallback = Nothing

    Public Sub New()
        ' initialize members
        MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY)
        ' Connect to the printer
        m_objmfDevice = New MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U")
        m_objmfBase = New MFBase
        m_objmfMicr = New MFMicr
        m_objmfScanFront = New MFScan
        m_objmfScanBack = New MFScan
        m_objmfPrint = New MFPrint
        m_objmfProcess = New MFProcess

        If (CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer" & ControlChars.NewLine & ControlChars.NewLine & _
                                "Ensure Driver installed correctly" & ControlChars.NewLine & "and Printer is powered on")) Then
            Exit Sub
        End If

        ' register for events
        ' Register the callback destination
        m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack()

        ' check for an error
        If (CheckResponse(m_errResultCode, "SCNMICRSetStatusBack")) Then
        End If
    End Sub

    ' this method is called when the user click the exit button
    Public Sub Close()
        If Not (m_objmfDevice Is Nothing) Then
            m_objmfDevice.SCNMICRCancelStatusBack()
            m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE)
            m_objmfDevice.CloseMonPrinter()
        End If
    End Sub

    ' this method is called when the user click the cancel button
    Public Sub CancelScan()
        If Not (m_objmfDevice Is Nothing) Then
            Call CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE), _
                                "SCNMICRCancelFunction(MF_EJECT_DISCHARGE)")
        End If
    End Sub

    ' this method is called when the user click the configure button
    Public Sub Configure()
        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
            Return
        End If

        ' ScanBack
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
            Return
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

    End Sub

    ' copy MianForm object
    Public Sub MainFormItem(ByVal Item As MainForm)
        m_objMainForm = Item
    End Sub

    ' this method is called when the user click the scan button
    Public Sub Scan()
        m_errResultCode = m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC)

        If (CheckResponse(m_errResultCode, "SCNMICRFunctionPostPrint(MF_EXEC)")) Then
            Complete(0)
        End If
    End Sub

    ' This method is called when the Complete scan
    Private Sub Complete(ByVal transactionNumber As Integer)
        If (m_objMainForm Is Nothing) Then
            Return
        End If
        m_objMainForm.ScanComplete(transactionNumber)
        m_objmfDevice.ResetLastError()
    End Sub

    ' This method is called whenever the scan state changes
    Private Sub SCNMICRSetStatusBack(ByVal transactionNumber As Integer, ByVal mainStatus As MainStatus, ByVal subStatus As ErrorCode, ByVal portName As String) Handles m_objmfDevice.SCNMICRStatusCallback
        ' Data receive complete
        If (mainStatus = mainStatus.MF_DATARECEIVE_DONE) Then
            Dim micr As String = ""

            ' Micr data
            Dim strErr As String = ""
            ' Specify the relevant item for MICR text
            m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR
            m_objmfMicr.Parsing = False
            ' Acquire MICR text
            m_objmfDevice.GetMicrText(transactionNumber, m_objmfMicr)
            micr = m_objmfMicr.MicrStr.Substring(0, m_objmfMicr.MicrStr.IndexOf(Nothing))
            If Not (m_objmfMicr.Ret = ErrorCode.SUCCESS) Then
                ' Add error code
                micr += " ( " & m_objmfMicr.Ret.ToString() & " ) "
            End If

            ' Front image
            ' Set the relevant item for the front side image
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)
            If (ErrorCode.SUCCESS = m_errResultCode) Then
                ' Specify the file format in BMP
                m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP)
                If (ErrorCode.SUCCESS = m_errResultCode) Then
                    ' Acquire image data
                    m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanFront)
                End If
            End If

            ' Back image
            ' Set the relevant item for the front side
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK)
            If (ErrorCode.SUCCESS = m_errResultCode) Then
                ' Specify the file format in BMP
                m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP)
                If (ErrorCode.SUCCESS = m_errResultCode) Then
                    ' Acquire image data
                    m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanBack)
                End If
            End If

            Update(m_objmfScanFront.Image, m_objmfScanBack.Image, micr)
        End If

        ' Finish scan process
        If (mainStatus = mainStatus.MF_FUNCTION_DONE) Then
            If Not (m_IAar Is Nothing) Then
                m_objMainForm.EndInvoke(m_IAar)
                m_IAar = Nothing
            End If

            Complete(transactionNumber)
        End If
    End Sub

    Delegate Sub UpdateCallback(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String)

    Private Sub Update(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String)
        If (m_objMainForm.InvokeRequired) Then
            If (m_UpdateCallback Is Nothing) Then
                m_UpdateCallback = New UpdateCallback(AddressOf Update)
            End If
            m_IAar = m_objMainForm.BeginInvoke(m_UpdateCallback, New Object() {FrontImage, BackImage, strMicr})
        Else
            m_objMainForm.SetFrontImage(FrontImage)
            m_objMainForm.SetBackImage(BackImage)
            m_objMainForm.SetMicr(strMicr)
        End If
    End Sub

    ' this method is return transaction number
    Public Function GetTransactionNumber() As Integer
        Return m_objmfDevice.TransactionNumber
    End Function

    ' this method is return response
    ' SUCCESS : Return to FALSE
    ' ERROR : Display the message box, return TRUE
    Private Function CheckResponse(ByVal errCode As ErrorCode, Optional ByVal str As String = "Error") As Boolean
        If Not (ErrorCode.SUCCESS.Equals(errCode)) Then
            MessageBox.Show(str, errCode.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
        End If
        Return False
    End Function

    Public Function GetStartWaitTime() As TimeSpan
        Dim startWaitTime As UInt32
        startWaitTime = m_objmfProcess.StartWaitTime
        If startWaitTime.CompareTo(Convert.ToUInt32(6400)) > 0 Then
            startWaitTime = Convert.ToUInt32(6400)
        End If
        Return TimeSpan.FromMilliseconds(Convert.ToDouble(startWaitTime))
    End Function

End Class
