Imports com.epson.bank.driver

Public Class ApiUsage
    Protected m_objMainForm As MainForm
    Protected m_objConfig As Properties = Nothing
    Dim WithEvents m_objmfDevice As MFDevice = Nothing
    Dim m_objmfBase As MFBase = Nothing
    Dim m_objmfMicr As MFMicr = Nothing
    Dim m_objmfScanFront As MFScan = Nothing
    Dim m_objmfScanBack As MFScan = Nothing
    Dim m_objmfPrint As MFPrint = Nothing
    Dim m_objmfProcess As MFProcess = Nothing
    Dim m_errResultCode As ErrorCode
    Dim m_IAar As System.IAsyncResult = Nothing
    Dim m_UpdateCallback As UpdateCallback = Nothing

    Public Sub New()
        ' initialize members
        MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY)
        ' Connect to the printer
        m_objmfDevice = New MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U")
        m_objmfBase = New MFBase
        m_objmfMicr = New MFMicr
        m_objmfScanFront = New MFScan
        m_objmfScanBack = New MFScan
        m_objmfPrint = New MFPrint
        m_objmfProcess = New MFProcess
        m_objConfig = New Properties

        If (CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer" & ControlChars.NewLine & ControlChars.NewLine & _
                                "Ensure Driver installed correctly" & ControlChars.NewLine & "and Printer is powered on")) Then
            Exit Sub
        End If

        ' register for events
        ' Register the callback destination
        m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack()

        ' check for an error
        If (CheckResponse(m_errResultCode, "BiSCNMICRSetStatusBack")) Then
        End If
    End Sub

    ' this method is called when the user click the exit button
    Public Sub Close()
        If Not (m_objmfDevice Is Nothing) Then
            m_objmfDevice.SCNMICRCancelStatusBack()
            m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE)
            m_objmfDevice.CloseMonPrinter()
        End If
    End Sub

    ' this method is called when the user click the cancel button
    Public Sub CancelScan()
        If Not (m_objmfDevice Is Nothing) Then
            Call CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE), _
                                "SCNMICRCancelFunction(MF_EJECT_DISCHARGE)")
        End If
    End Sub

    ' this method is called when the user click the configure button
    Public Sub Configure(ByVal proc As Properties)
        ' DPI setting method
        ' Specify MF_SCAN_DPI_XXX in sResolution of the MF_SCAN structure
        ' Execute BiSCNMICRFunctionXXX
        ' * Execute before scanning

        ' Sharpening setting method
        ' Specify EPS_BI_SCN_XXX as the 5th parameter of BiSCNSetImageQuality and 	execute it (other parameters are discretionary)
        ' * Execute before scanning

        ' Setting method for multiple electronic endorsements
        ' Execute MF_PRT_BUFFERING with BiBufferedPrint
        ' Set multiple electronic endorsements
        ' Execute MF_PRT_EXEC with BiBufferedPrint
        ' * Execute before scanning

        m_objConfig = proc

        ' Set station electronic endorse
        If (CheckResponse(m_objmfDevice.SetPrintStation(PrintingStation.MF_ST_E_ENDORSEMENT), "SetPrintStation(MF_ST_E_ENDORSEMENT)")) Then
            Return
        End If

        ' the registered electronic endorsement
        If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_CLEAR), "BufferedPrint(MF_PRT_CLEAR)")) Then
            Return
        End If

        Select Case proc("Scan Func")
            Case 0
                ConfigureMulti()
            Case 1
                ConfigureSingle()
            Case Else
                ConfigureMulti()
        End Select
    End Sub

    ' this method is settings of multi scan
    ' primarily the BiSCNMICRFunctionContinuously method is used
    Public Sub ConfigureMulti()

        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
            Return
        End If

        ' ScanBack
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
            Return
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

        ' Set the endorsement
        If (m_objConfig("Electronic Endorse Image")) Then
            ' Set the electronic endorsement
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), "BiBufferedPrint(MF_PRT_BUFFERING)")) Then
                Return
            End If

            ' Set the image electronic endorsement
            ElectricEndorseImage()

            ' Register the electronic endorsement set
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC), "BiBufferedPrint(MF_PRT_EXEC)")) Then
                Return
            End If
        End If

    End Sub

    ' this method is settings of single scan
    ' primarily the BiSCNMICRFunctionPostPrint method is used
    Public Sub ConfigureSingle()

        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
            Return
        End If

        ' ScanBack
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
            Return
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Operational settings when an error occurs
        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

        ' Set the endorsement
        If (m_objConfig("Electronic Endorse Image")) Then
            ' Set the electronic endorsement
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), "BiBufferedPrint(MF_PRT_BUFFERING)")) Then
                Return
            End If

            ' Set the image electronic endorsement
            ElectricEndorseImage()

            ' Register the electronic endorsement set
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC), "BiBufferedPrint(MF_PRT_EXEC)")) Then
                Return
            End If
        End If

    End Sub

    ' copy MianForm object
    Public Sub MainFormItem(ByVal Item As MainForm)
        m_objMainForm = Item
    End Sub

    ' this method is called when the user click the scan button
    Public Sub Scan()
        Select Case m_objConfig("Scan Func")
            Case 0
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC), _
                                    "BiSCNMICRFunctionContinuously(MF_EXEC)")) Then
                    Complete(0)
                End If

            Case 1
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC), _
                                    "BiSCNMICRFunctionPostPrint(MF_EXEC)")) Then
                    Complete(0)
                End If

            Case Else
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC), _
                                    "BiSCNMICRFunctionContinuously(MF_EXEC)")) Then
                    Complete(0)
                End If

        End Select

    End Sub

    ' This method is called when the Complete scan
    Private Sub Complete(ByVal transactionNumber As Integer)
        If (m_objMainForm Is Nothing) Then
            Return
        End If
        m_objMainForm.ScanComplete(transactionNumber)
        m_objmfDevice.ResetLastError()
    End Sub

    ' This method is called whenever the scan state changes
    ' Called when scanning with BiSCNMICRFunctionPostPrint/BiSCNMICRFunctionContinuously
    Private Sub SCNMICRSetStatusBack(ByVal transactionNumber As Integer, ByVal mainStatus As MainStatus, ByVal subStatus As ErrorCode, ByVal portName As String) Handles m_objmfDevice.SCNMICRStatusCallback
        ' Data receive complete
        If (mainStatus = mainStatus.MF_DATARECEIVE_DONE) Then
            Dim micr As String = ""

            ' Micr data
            ' Specify the relevant item for MICR text
            m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR
            m_objmfMicr.Parsing = False
            ' Acquire MICR text
            m_objmfDevice.GetMicrText(transactionNumber, m_objmfMicr)
            micr = m_objmfMicr.MicrStr.Substring(0, m_objmfMicr.MicrStr.IndexOf(Nothing))
            If Not (m_objmfMicr.Ret = ErrorCode.SUCCESS) Then
                ' Add error code
                micr += " ( " + m_objmfMicr.Ret.ToString() + " ) "
            End If

            ' Front image
            ' Set the relevant item for the front side image
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)
            If (m_errResultCode = ErrorCode.SUCCESS) Then
                ' Specify the file format in BMP
                m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP)
                If (m_errResultCode = ErrorCode.SUCCESS) Then
                    ' Acquire image data
                    m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanFront)
                End If
            End If

            ' Electric endorse
            If (m_objConfig("Electronic Endorse Text")) Then
                ElectricEndorseText()
            End If

            ' Back image
            ' Set the relevant item for the front side
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK)
            If (m_errResultCode = ErrorCode.SUCCESS) Then
                ' Specify the file format in BMP
                m_errResultCode = m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_BITMAP)
                If (m_errResultCode = ErrorCode.SUCCESS) Then
                    ' Acquire image data
                    m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanBack)
                End If
            End If

            Update(m_objmfScanFront.Image, m_objmfScanBack.Image, micr)

            ' Finish scan process
        ElseIf (mainStatus = mainStatus.MF_FUNCTION_DONE) Then
            If Not (m_IAar Is Nothing) Then
                m_objMainForm.EndInvoke(m_IAar)
                m_IAar = Nothing
            End If

            Complete(transactionNumber)
        End If
    End Sub

    ' This delegate enables asynchronous calls for setting.
    Delegate Sub UpdateCallback(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String)

    Private Sub Update(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String)
        If (m_objMainForm.InvokeRequired) Then
            If (m_UpdateCallback Is Nothing) Then
                m_UpdateCallback = New UpdateCallback(AddressOf Update)
            End If
            m_IAar = m_objMainForm.BeginInvoke(m_UpdateCallback, New Object() {FrontImage, BackImage, strMicr})
        Else
            m_objMainForm.SetFrontImage(FrontImage)
            m_objMainForm.SetBackImage(BackImage)
            m_objMainForm.SetMicr(strMicr)
        End If
    End Sub

    ' this method is return transaction number
    Public Function GetTransactionNumber() As Integer
        Return m_objmfDevice.TransactionNumber
    End Function

    ' settings endorse text
    Private Sub ElectricEndorseText()
        Dim mfDecorate As MFTrueType = Nothing
        Dim objFont As System.Drawing.Font
        m_objmfDevice.ResetLastError()


        mfDecorate = New MFTrueType
        mfDecorate.Color = PrintColor.MF_PRINT_BLACK
        objFont = New System.Drawing.Font("Arial", 30)
        mfDecorate.Font = objFont
        ' set position
        If (CheckResponse(m_objmfDevice.SetPrintPosition(10, 60), "BiSetPrintPosition(10, 60)")) Then
            Return
        End If
        m_errResultCode = m_objmfDevice.PrintText("Transaction#:<00000000>", mfDecorate)
        If (CheckResponse(m_errResultCode, "BiPrintText")) Then
            Return
        End If
    End Sub

    ' settings endorse image
    Private Sub ElectricEndorseImage()
        ' set position
        If (CheckResponse(m_objmfDevice.SetPrintPosition(100, 30), "BiSetPrintPosition(100, 30)")) Then
            Return
        End If
        ' set size
        If (CheckResponse(m_objmfDevice.SetPrintSize(30, 30), "BiSetPrintSize(30, 30)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.PrintImage("Image.jpg"), "BiPrintImage(Image.jpg)")) Then
            Return
        End If
    End Sub

    ' this method is return response
    ' SUCCESS : Return to FALSE
    ' ERROR : Display the message box, return TRUE
    Private Function CheckResponse(ByVal errCode As ErrorCode, Optional ByVal str As String = "Error") As Boolean
        If Not (ErrorCode.SUCCESS.Equals(errCode)) Then
            MessageBox.Show(str, errCode.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
        End If
        Return False
    End Function

    Public Function GetStartWaitTime() As TimeSpan
        Dim startWaitTime As UInt32
        startWaitTime = m_objmfProcess.StartWaitTime
        If startWaitTime.CompareTo(Convert.ToUInt32(6400)) > 0 Then
            startWaitTime = Convert.ToUInt32(6400)
        End If
        Return TimeSpan.FromMilliseconds(Convert.ToDouble(startWaitTime))
    End Function
End Class
