Imports com.epson.bank.driver

Public Class ApiUsage
    Protected m_objMainForm As MainForm = Nothing
    Protected m_objConfig As Properties = Nothing
    Protected m_bNearFullFlg As Boolean = False
    Protected m_NearFullThread As System.Threading.Thread = Nothing
    Protected m_Thread As System.Threading.WaitCallback = Nothing
    Dim WithEvents m_objmfDevice As MFDevice = Nothing
    Dim m_objmfBase As MFBase = Nothing
    Dim m_objmfMicr As MFMicr = Nothing
    Dim m_objmfScanFront As MFScan = Nothing
    Dim m_objmfScanBack As MFScan = Nothing
    Dim m_objmfPrint As MFPrint = Nothing
    Dim m_objmfProcess As MFProcess = Nothing
    Dim m_objmfIqa As MFIqa = Nothing
    Dim m_objmfBarcodeFront As MFBarcode = Nothing
    Dim m_objmfBarcodeBack As MFBarcode = Nothing
    Dim m_errResultCode As ErrorCode
    Dim m_errErrorOccured As ErrorCode
    Dim m_IAar As System.IAsyncResult = Nothing
    Dim m_UpdateCallback As UpdateCallback = New UpdateCallback(AddressOf Update)
    Dim m_NearFullMessageCallback As NearFullMessageCallback = New NearFullMessageCallback(AddressOf NearFullMessage)
    Dim m_RemoveMessageCallback As RemoveMessageCallback = New RemoveMessageCallback(AddressOf RemoveMessage)
    Dim m_bEndScanProcess As Boolean = False

    Protected Const OCR_AB_START_X As Integer = 30
    Protected Const OCR_AB_START_Y As Integer = 56
    Protected Const OCR_AB_END_X As Integer = 110
    Protected Const OCR_AB_END_Y As Integer = 70

    Public Sub New()
        ' initialize members
        MFDevice.ESCNEnable(Storage.CROP_STORE_MEMORY)
        ' Connect to the printer
        m_objmfDevice = New MFDevice(OpenType.TYPE_PRINTER, "TM-S1000U")
        m_objmfBase = New MFBase
        m_objmfMicr = New MFMicr
        m_objmfScanFront = New MFScan
        m_objmfScanBack = New MFScan
        m_objmfPrint = New MFPrint
        m_objmfProcess = New MFProcess
        m_objmfIqa = New MFIqa
        m_objmfBarcodeFront = New MFBarcode
        m_objmfBarcodeBack = New MFBarcode
        m_objConfig = New Properties

        If (CheckResponse(m_objmfDevice.LastError, "Unable to connect to printer" & ControlChars.NewLine & ControlChars.NewLine & _
                                "Ensure Driver installed correctly" & ControlChars.NewLine & "and Printer is powered on")) Then
            Exit Sub
        End If

        ' register for events
        ' Register the callback destination
        m_errResultCode = m_objmfDevice.SCNMICRSetStatusBack()

        ' check for an error
        If (CheckResponse(m_errResultCode, "SCNMICRSetStatusBack")) Then
            Exit Sub
        End If

        If (CheckResponse(m_objmfDevice.SetStatusBack(), "SetStatusBack()")) Then
        End If

        m_objmfDevice.ResetLastError()

        m_Thread = New System.Threading.WaitCallback(AddressOf NearFullMessage)
    End Sub

    ' this method is called when the user click the exit button
    Public Sub Close()
        If Not (m_objmfDevice Is Nothing) Then
            m_objmfDevice.SCNMICRCancelStatusBack()
            m_objmfDevice.CancelStatusBack()
            m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE)
            m_objmfDevice.CloseMonPrinter()
        End If
    End Sub

    ' this method is called when the user click the cancel button
    Public Sub CancelScan()
        If Not (m_objmfDevice Is Nothing) Then
            Call CheckResponse(m_objmfDevice.SCNMICRCancelFunction(MfEjectType.MF_EJECT_DISCHARGE), _
                                "SCNMICRCancelFunction(MF_EJECT_DISCHARGE)")
        End If
    End Sub

    ' this method is called when the user click the configure button
    Public Sub Configure(ByVal proc As Properties)
        ' DPI setting method
        ' Specify MF_SCAN_DPI_XXX in sResolution of the MF_SCAN structure
        ' Execute BiSCNMICRFunctionXXX
        ' * Execute before scanning

        ' Sharpening setting method
        ' Specify EPS_BI_SCN_XXX as the 5th parameter of BiSCNSetImageQuality and 	execute it (other parameters are discretionary)
        ' * Execute before scanning

        ' Setting method for multiple electronic endorsements
        ' Execute MF_PRT_BUFFERING with BiBufferedPrint
        ' Set multiple electronic endorsements
        ' Execute MF_PRT_EXEC with BiBufferedPrint
        ' * Execute before scanning
        Dim eWaterfallMode As WaterfallMode = WaterfallMode.WATERFALL_MODE_DISABLE

        SetProc(proc)

        ' Set station electronic endorse
        If (CheckResponse(m_objmfDevice.SetPrintStation(PrintingStation.MF_ST_E_ENDORSEMENT), "SetPrintStation(MF_ST_E_ENDORSEMENT)")) Then
            Return
        End If

        ' the registered electronic endorsement
        If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_CLEAR), "BufferedPrint(MF_PRT_CLEAR)")) Then
            Return
        End If

        Select Case proc(Properties.SCAN_FUNC)
            Case 0
                ConfigureMulti()
            Case 1
                ConfigureSingle()
            Case Else
                ConfigureMulti()
        End Select

        If (m_objConfig(Properties.WATERFALL_ENABLE)) Then
            If (m_objConfig(Properties.WATERFALL_MODE) = 0) Then
                eWaterfallMode = WaterfallMode.WATERFALL_MODE_STANDARD
            Else
                eWaterfallMode = WaterfallMode.WATERFALL_MODE_INHERIT_POCKET
            End If
        End If
        ' Set Waterfall mode
        If (CheckResponse(m_objmfDevice.SetWaterfallMode(eWaterfallMode), "SetWaterfallMode(eWaterfallMode)")) Then
            Return
        End If
    End Sub

    ' this method is settings of multi scan
    ' primarily the BiSCNMICRFunctionContinuously method is used
    Private Sub ConfigureMulti()

        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        SetBaseStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        ' MICR font designation
        Select Case (m_objConfig(Properties.MICR_FONT))
            Case 0
                m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_E13B
            Case 1
                m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_CMC7
        End Select
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (m_objConfig(Properties.SCAN_FRONT)) Then
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                                "SCNMICRFunctionContinuously(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
                Return
            End If
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                                "SCNMICRFunctionContinuously(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
                Return
            End If
        Else
            ' Delete the structure set in MF_CLEAR_SCAN_FRONT_PARAM when the front side will not be scanned
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_CLEAR_SCAN_FRONT_PARAM), _
                                "SCNMICRFunctionContinuously(MF_CLEAR_SCAN_FRONT_PARAM)")) Then
                Return
            End If
        End If

        ' ScanBack
        If (m_objConfig(Properties.SCAN_BACK)) Then
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                                "SCNMICRFunctionContinuously(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
                Return
            End If
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                                "SCNMICRFunctionContinuously(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
                Return
            End If
        Else
            ' Delete the structure set in MF_CLEAR_SCAN_BACK_PARAM when the reverse side will not be scanned
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_CLEAR_SCAN_BACK_PARAM), _
                                 "SCNMICRFunctionContinuously(MF_CLEAR_SCAN_BACK_PARAM)")) Then
                Return
            End If
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Operational settings when an error occurs
        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionContinuously(m_objmfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        SetProcessStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionContinuously(m_objmfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

        ' Set the endorsement
        If (m_objConfig(Properties.ELEC_ENDORSE_IMAGE)) Then
            ' Set the electronic endorsement
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), "BiBufferedPrint(MF_PRT_BUFFERING)")) Then
                Return
            End If

            ' Set the image electronic endorsement
            ElectricEndorseImage()

            ' Register the electronic endorsement set
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC), "BiBufferedPrint(MF_PRT_EXEC)")) Then
                Return
            End If
        End If

        ' Iqa
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfIqa, FunctionType.MF_GET_IQA_DEFAULT), _
         "SCNMICRFunctionContinuously(m_objmfIqa, MF_GET_IQA_DEFAULT)")) Then
            Return
        End If
        SetIqaStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfIqa, FunctionType.MF_SET_IQA_PARAM), _
         "SCNMICRFunctionContinuously(m_objmfIqa, MF_SET_IQA_PARAM)")) Then
            Return
        End If

        ' Barcode
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeFront, FunctionType.MF_GET_BARCODE_FRONT_DEFAULT), _
         "SCNMICRFunctionContinuously(m_objmfBarcodeFront, MF_GET_BARCODE_FRONT_DEFAULT)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeBack, FunctionType.MF_GET_BARCODE_BACK_DEFAULT), _
         "SCNMICRFunctionContinuously(m_objmfBarcodeBack, MF_GET_BARCODE_BACK_DEFAULT)")) Then
            Return
        End If
        SetBarcodeStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeFront, FunctionType.MF_SET_BARCODE_FRONT_PARAM), _
         "SCNMICRFunctionContinuously(m_objmfBarcodeFront, MF_SET_BARCODE_FRONT_PARAM)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(m_objmfBarcodeBack, FunctionType.MF_SET_BARCODE_BACK_PARAM), _
         "SCNMICRFunctionContinuously(m_objmfBarcodeBack, MF_SET_BARCODE_BACK_PARAM)")) Then
            Return
        End If
    End Sub

    ' this method is settings of single scan
    ' primarily the BiSCNMICRFunctionPostPrint method is used
    Private Sub ConfigureSingle()

        ' Base
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_GET_BASE_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_GET_BASE_DEFAULT)")) Then
            Return
        End If
        SetBaseStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfBase, FunctionType.MF_SET_BASE_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfBase, MF_SET_BASE_PARAM)")) Then
            Return
        End If

        ' MICR
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_GET_MICR_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_GET_MICR_DEFAULT)")) Then
            Return
        End If
        ' MICR font designation
        Select Case (m_objConfig(Properties.MICR_FONT))
            Case 0
                m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_E13B
            Case 1
                m_objmfMicr.Font = MfMicrFont.MF_MICR_FONT_CMC7
        End Select
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfMicr, FunctionType.MF_SET_MICR_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfMicr, MF_SET_MICR_PARAM)")) Then
            Return
        End If

        ' ScanFront
        If (m_objConfig(Properties.SCAN_FRONT)) Then
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_GET_SCAN_FRONT_DEFAULT), _
                                "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_GET_SCAN_FRONT_DEFAULT)")) Then
                Return
            End If
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanFront, FunctionType.MF_SET_SCAN_FRONT_PARAM), _
                                "SCNMICRFunctionPostPrint(m_objmfScanFront, MF_SET_SCAN_FRONT_PARAM)")) Then
                Return
            End If
        Else
            ' Delete the structure set in MF_CLEAR_SCAN_FRONT_PARAM when the front side will not be scanned
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_CLEAR_SCAN_FRONT_PARAM), _
                                "SCNMICRFunctionPostPrint(MF_CLEAR_SCAN_FRONT_PARAM)")) Then
                Return
            End If
        End If

        ' ScanBack
        If (m_objConfig(Properties.SCAN_BACK)) Then
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_GET_SCAN_BACK_DEFAULT), _
                                "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_GET_SCAN_BACK_DEFAULT)")) Then
                Return
            End If
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfScanBack, FunctionType.MF_SET_SCAN_BACK_PARAM), _
                                "SCNMICRFunctionPostPrint(m_objmfScanBack, MF_SET_SCAN_BACK_PARAM)")) Then
                Return
            End If
        Else
            ' Delete the structure set in MF_CLEAR_SCAN_BACK_PARAM when the reverse side will not be scanned
            If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_CLEAR_SCAN_BACK_PARAM), _
                                "SCNMICRFunctionPostPrint(MF_CLEAR_SCAN_BACK_PARAM)")) Then
                Return
            End If
        End If

        ' Print
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_GET_PRINT_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_GET_PRINT_DEFAULT)")) Then
            Return
        End If
        ' An error when the m_objmfPrint.String array is all NULL
        m_objmfPrint.String(EndorsementLine.MF_PRINT_LINE_1) = "" ' Specify blank characters
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfPrint, FunctionType.MF_SET_PRINT_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfPrint, MF_SET_PRINT_PARAM)")) Then
            Return
        End If

        ' Operational settings when an error occurs
        ' Process
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_GET_PROCESS_DEFAULT), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_GET_PROCESS_DEFAULT)")) Then
            Return
        End If
        SetProcessStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfProcess, FunctionType.MF_SET_PROCESS_PARAM), _
                            "SCNMICRFunctionPostPrint(m_objmfProcess, MF_SET_PROCESS_PARAM)")) Then
            Return
        End If

        ' Set the endorsement
        If (m_objConfig(Properties.ELEC_ENDORSE_IMAGE)) Then
            ' Set the electronic endorsement
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_BUFFERING), "BiBufferedPrint(MF_PRT_BUFFERING)")) Then
                Return
            End If

            ' Set the image electronic endorsement
            ElectricEndorseImage()

            ' Register the electronic endorsement set
            If (CheckResponse(m_objmfDevice.BufferedPrint(PrintBuffer.MF_PRT_EXEC), "BiBufferedPrint(MF_PRT_EXEC)")) Then
                Return
            End If
        End If

        ' Iqa
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfIqa, FunctionType.MF_GET_IQA_DEFAULT), _
         "SCNMICRFunctionPostPrint(m_objmfIqa, MF_GET_IQA_DEFAULT)")) Then
            Return
        End If
        SetIqaStruct()
        If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(m_objmfIqa, FunctionType.MF_SET_IQA_PARAM), _
         "SCNMICRFunctionPostPrint(m_objmfIqa, MF_SET_IQA_PARAM)")) Then
            Return
        End If
    End Sub

    ' copy MianForm object
    Public Sub MainFormItem(ByVal Item As MainForm)
        m_objMainForm = Item
    End Sub

    ' this method is called when the user click the scan button
    Public Sub Scan()
        m_bNearFullFlg = False
        m_bEndScanProcess = False
        Select Case m_objConfig(Properties.SCAN_FUNC)
            Case 0
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC), _
                                        "BiSCNMICRFunctionContinuously(MF_EXEC)")) Then
                    Complete(0)
                End If
            Case 1
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionPostPrint(FunctionType.MF_EXEC), _
                                        "BiSCNMICRFunctionPostPrint(MF_EXEC)")) Then
                    Complete(0)
                End If
            Case Else
                If (CheckResponse(m_objmfDevice.SCNMICRFunctionContinuously(FunctionType.MF_EXEC), _
                                        "BiSCNMICRFunctionContinuously(MF_EXEC)")) Then
                    Complete(0)
                End If

        End Select

    End Sub

    ' this method is insert proc in m_objConfig
    Public Sub SetProc(ByVal proc As Properties)
        m_objConfig = proc
    End Sub

    ' This method is called when the Complete scan
    Private Sub Complete(ByVal transactionNumber As Integer)
        If (m_objMainForm Is Nothing) Then
            Return
        End If
        m_objMainForm.ScanComplete(transactionNumber)
        m_objmfDevice.ResetLastError()
    End Sub


    ' This method is called whenever the scan state changes
    ' Called when scanning with BiSCNMICRFunctionPostPrint/BiSCNMICRFunctionContinuously
    Private Sub SCNMICRSetStatusBack(ByVal transactionNumber As Integer, ByVal mainStatus As MainStatus, ByVal subStatus As ErrorCode, ByVal portName As String) Handles m_objmfDevice.SCNMICRStatusCallback

        ' Data receive complete
        If (mainStatus = mainStatus.MF_DATARECEIVE_DONE) Then

            Dim micr As String = ""
            Dim ocrAB As String = ""
            Dim FrontDisplay As Boolean = False
            Dim FrontGrayscale As Boolean = False
            Dim BackDisplay As Boolean = False
            Dim BackGrayscale As Boolean = False
            Dim mfIqaResult As MFIqaResult = Nothing

            ' Micr data
            If (m_objConfig(Properties.MICR)) Then
                ' Specify the relevant item for MICR text
                m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR
                m_objmfMicr.Parsing = False
                ' Acquire MICR text
                m_objmfDevice.GetMicrText(transactionNumber, m_objmfMicr)
                micr = m_objmfMicr.MicrStr.Substring(0, m_objmfMicr.MicrStr.IndexOf(Nothing))
                If Not (m_objmfMicr.Ret = ErrorCode.SUCCESS) Then
                    ' Add error code
                    micr += " ( " & m_objmfMicr.Ret.ToString & " ) "
                End If
            End If

            If (m_objConfig(Properties.OCR_AB) And m_objConfig(Properties.SCAN_FRONT)) Then
                ' Ocr AB
                Dim ocrAB_Font As String = ""
                Dim mfOcrAb As MFOcrAb = New MFOcrAb
                ' Font designation
                Select Case (m_objConfig(Properties.OCR_AB_FONT))
                    Case 0
                        mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRA_ALPHANUM
                        ocrAB_Font = "OCR_A:"
                    Case 1
                        mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRB_ALPHANUM
                        ocrAB_Font = "OCR_B:"
                End Select

                mfOcrAb.Direction = MfDirection.MF_OCR_LEFTRIGHT
                mfOcrAb.StartX = Convert.ToUInt16(OCR_AB_START_X)
                mfOcrAb.StartY = Convert.ToUInt16(OCR_AB_START_Y)
                mfOcrAb.EndX = Convert.ToUInt16(OCR_AB_END_X)
                mfOcrAb.EndY = Convert.ToUInt16(OCR_AB_END_Y)
                mfOcrAb.SpeceHandling = MfSpeceHandling.OCR_SPACE_ENABLE

                ' select front side image
                m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)

                ' Acquire OCR_AB text
                m_errResultCode = m_objmfDevice.GetOcrABText(Convert.ToUInt32(transactionNumber), OcrImageSource.OCR_SOURCE_TRANSACTION_NUMBER, "", mfOcrAb)
                ocrAB = ocrAB_Font & mfOcrAb.OcrStr.Substring(0, mfOcrAb.OcrStr.IndexOf(Nothing))
                If Not (mfOcrAb.Ret = ErrorCode.SUCCESS) Then
                    ' Add error code
                    ocrAB += " ( " & mfOcrAb.Ret.ToString() & " ) "
                End If
            End If

            ' Save MICR data
            If (m_objConfig(Properties.MICR_SAVE) And m_objConfig(Properties.MICR_SAVE_ENABLE)) Then
                MicrSave(transactionNumber)
            End If

            ' Front image
            If (m_objConfig(Properties.SCAN_FRONT)) Then
                Dim ColorDepth As ColorDepth

                ' Set the relevant item for the front side image
                m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)
                If (ErrorCode.SUCCESS.Equals(m_errResultCode)) Then
                    ' Set grayscale
                    FrontGrayscale = m_objConfig(Properties.FRONT_GRAYSCALE)
                    If (FrontGrayscale) Then
                        ColorDepth = ColorDepth.EPS_BI_SCN_8BIT
                    Else
                        ColorDepth = ColorDepth.EPS_BI_SCN_1BIT
                    End If
                    m_errResultCode = m_objmfDevice.SCNSetImageQuality(ColorDepth, 0, Color.EPS_BI_SCN_MONOCHROME, ExOption.EPS_BI_SCN_SHARP)
                End If

                ' Set format
                If (FrontGrayscale) Then
                    m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_JPEGNORMAL)
                Else
                    m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_TIFF)
                End If

                ' Acquire image data
                m_errResultCode = m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanFront)

                ' Display front image
                FrontDisplay = m_objConfig(Properties.FRONT_DISPLAY)

                ' Save front iamge
                If (m_objConfig(Properties.FRONT_SAVE)) Then
                    Dim filename As String = ""
                    ' Set grayscale
                    If (FrontGrayscale) Then
                        filename = transactionNumber.ToString("000") + ".front.jpg"
                    Else
                        filename = transactionNumber.ToString("000") + ".front.tif"
                    End If

                    If (ErrorCode.SUCCESS.Equals(m_errResultCode)) Then
                        Dim data As System.IO.Stream = m_objmfScanFront.Data
                        Dim file As System.IO.FileStream = New System.IO.FileStream(filename, IO.FileMode.Create)
                        Dim buffer(data.Length) As Byte

                        ' Save in file
                        data.Read(buffer, 0, buffer.Length)
                        file.Write(buffer, 0, buffer.Length)
                        file.Close()
                        data.Close()
                    End If
                End If
            End If

            ' Electric endorse is settings
            If (m_objConfig(Properties.ELEC_ENDORSE_TEXT)) Then
                ElectricEndorseText()
            End If

            ' Back image
            If (m_objConfig(Properties.SCAN_BACK)) Then
                Dim ColorDepth As ColorDepth

                ' Set the relevant item for the front side
                m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK)
                If (ErrorCode.SUCCESS.Equals(m_errResultCode)) Then
                    ' Set grayscale
                    BackGrayscale = m_objConfig(Properties.BACK_GRAYSCALE)
                    If (BackGrayscale) Then
                        ColorDepth = ColorDepth.EPS_BI_SCN_8BIT
                    Else
                        ColorDepth = ColorDepth.EPS_BI_SCN_1BIT
                    End If
                    m_errResultCode = m_objmfDevice.SCNSetImageQuality(ColorDepth, 0, Color.EPS_BI_SCN_MONOCHROME, ExOption.EPS_BI_SCN_SHARP)
                End If

                ' Set format
                If (BackGrayscale) Then
                    m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_JPEGNORMAL)
                Else
                    m_objmfDevice.SCNSetImageFormat(Format.EPS_BI_SCN_TIFF)
                End If

                ' Acquire image data
                m_errResultCode = m_objmfDevice.GetScanImage(transactionNumber, m_objmfScanBack)

                ' Display back image
                BackDisplay = m_objConfig(Properties.BACK_DISPLAY)

                ' Save back iamge
                If (m_objConfig(Properties.BACK_SAVE)) Then
                    Dim filename As String = ""
                    ' Set grayscale
                    If (BackGrayscale) Then
                        filename = transactionNumber.ToString("000") + ".back.jpg"
                    Else
                        filename = transactionNumber.ToString("000") + ".back.tif"
                    End If

                    If (ErrorCode.SUCCESS.Equals(m_errResultCode)) Then
                        Dim data As System.IO.Stream = m_objmfScanBack.Data
                        Dim file As System.IO.FileStream = New System.IO.FileStream(filename, IO.FileMode.Create)
                        Dim buffer(data.Length) As Byte

                        ' Save in file
                        data.Read(buffer, 0, buffer.Length)
                        file.Write(buffer, 0, buffer.Length)
                        file.Close()
                        data.Close()
                    End If
                End If
            End If

            If ((FrontDisplay = True) AndAlso (BackDisplay = True)) Then
                Update(m_objmfScanFront.Image, m_objmfScanBack.Image, micr, ocrAB)
            ElseIf ((FrontDisplay = False) AndAlso (BackDisplay = True)) Then
                Update(Nothing, m_objmfScanBack.Image, micr, ocrAB)
            ElseIf ((FrontDisplay = True) AndAlso (BackDisplay = False)) Then
                Update(m_objmfScanFront.Image, Nothing, micr, ocrAB)
            Else
                Update(Nothing, Nothing, micr, ocrAB)
            End If

            If (m_objConfig(Properties.IQA_DETECT)) Then
                mfIqaResult = New MFIqaResult
                m_objmfDevice.GetIqaResult(transactionNumber, mfIqaResult)
                m_objMainForm.SetIqaResult(mfIqaResult)
            End If

            ' display barcode text
            DisplayBarcodeData(transactionNumber)

            ' Is it Confirmation Mode?
            If (m_objConfig(Properties.CONFIRMATION)) Then
                ' Execute BiSetBehaviorToScnResult?
                If (m_objConfig(Properties.RUN_SCN_TO_RESULT)) Then
                    ' Call BiSetBehaviorToScnResult?
                    If Not (m_objConfig(Properties.NO_CALL_SCN_TO_RESULT)) Then
                        ' Display BiSetBehaviorToScnResult dialog box
                        ' Do no execute if it is canceled in the dialog box
                        If (m_objMainForm.Confirmation()) Then
                            Dim NextCheck As MfProcessContinue
                            Select Case (m_objConfig(Properties.CONF_NEXT_CHECK))
                                Case 0
                                    NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP
                                Case 1
                                    NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_NOOVERLAP
                                Case 2
                                    NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_CANCEL
                            End Select
                            m_objmfDevice.SetBehaviorToScnResult(GetEject(m_objConfig(Properties.CONF_EJECT)), GetStamp(m_objConfig(Properties.CONF_STAMP)), NextCheck)
                        End If
                    Else
                        ' Use the value set without calling BiSetBehaviorToScnResult
                        Dim NextCheck As MfProcessContinue
                        Select Case (m_objConfig(Properties.CONF_NEXT_CHECK))
                            Case 0
                                NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_OVERLAP
                            Case 1
                                NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_NOOVERLAP
                            Case 2
                                NextCheck = MfProcessContinue.MF_PROCESS_CONTINUE_CANCEL
                        End Select
                        m_objmfDevice.SetBehaviorToScnResult(GetEject(m_objConfig(Properties.CONF_EJECT)), GetStamp(m_objConfig(Properties.CONF_STAMP)), NextCheck)
                    End If
                End If
                m_errErrorOccured = ErrorCode.SUCCESS
            End If
            ' Finish scan process
        ElseIf (mainStatus = mainStatus.MF_FUNCTION_DONE) Then
            If Not (m_IAar Is Nothing) Then
                m_objMainForm.EndInvoke(m_IAar)
                m_IAar = Nothing
            End If

            m_bEndScanProcess = True

            Complete(transactionNumber)

            If (subStatus = ErrorCode.ERR_PAPER_JAM) Then
                RemoveMessage()

                ' Cancel mechanical error
                m_objmfDevice.CancelError()
            ElseIf Not (subStatus = ErrorCode.SUCCESS Or subStatus = ErrorCode.ERR_PAPER_JAM) Then
                MessageBox.Show(subStatus.ToString(), "Error")
            End If
            ' Error Occured
        ElseIf (mainStatus = mainStatus.MF_ERROR_OCCURED) Then
            m_errErrorOccured = subStatus
            ' NearFull Error
        ElseIf (mainStatus = mainStatus.MF_CHECKPAPER_PROCESS_DONE) Then
            Dim Status As Integer = GetStatus()
            If Not (m_bNearFullFlg) Then
                ' Near full error
                If Not (Convert.ToBoolean(Status And ASB.ASB_MAIN_NEAR_FULL) AndAlso Convert.ToBoolean(Status And ASB.ASB_SUB_NEAR_FULL)) Then
                    m_bNearFullFlg = True
                    System.Threading.ThreadPool.QueueUserWorkItem(m_Thread)
                End If
            Else
                ' No Near full error
                If (Convert.ToBoolean(Status And ASB.ASB_MAIN_NEAR_FULL) And Convert.ToBoolean(Status And ASB.ASB_SUB_NEAR_FULL)) Then
                    m_bNearFullFlg = False
                    System.Threading.ThreadPool.QueueUserWorkItem(m_Thread)
                End If
            End If
        End If
    End Sub

    ' This delegate enables asynchronous calls for setting.
    Delegate Sub NearFullMessageCallback(ByVal state As Object)

    ' Display Near Full message
    Public Sub NearFullMessage(ByVal state As Object)
        If (m_objMainForm.InvokeRequired) Then
            If (m_NearFullMessageCallback Is Nothing) Then
                m_NearFullMessageCallback = New NearFullMessageCallback(AddressOf NearFullMessage)
            End If
            m_objMainForm.Invoke(m_NearFullMessageCallback, New Object() {state})
        Else
            If (m_bEndScanProcess) Then
                Exit Sub
            End If

            If (m_bNearFullFlg) Then
                m_objmfDevice.RingBuzzer(MfBuzzerTone.MF_BUZZER_TONE_HIGH, 3, Convert.ToUInt16(1), Convert.ToUInt16(0))
                m_objMainForm.Text = "TM-S1000SampleStep8  -  ***** NearFull *****"
            Else
                m_objMainForm.Text = "TM-S1000SampleStep8"
            End If
        End If
    End Sub

    ' This delegate enables asynchronous calls for setting.
    Delegate Sub RemoveMessageCallback()

    Private Sub RemoveMessage()
        If (m_objMainForm.InvokeRequired) Then
            If (m_RemoveMessageCallback Is Nothing) Then
                m_RemoveMessageCallback = New RemoveMessageCallback(AddressOf RemoveMessage)
            End If
            m_objMainForm.Invoke(m_RemoveMessageCallback, New Object() {})
        Else
            ' Display the prompt "Remove the paper".
            MessageBox.Show(m_objMainForm, "Remove the paper", "MECHANICAL ERROR", MessageBoxButtons.OK)
        End If
    End Sub

    ' This delegate enables asynchronous calls for setting.
    Delegate Sub UpdateCallback(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String, ByVal strOcr As String)

    Private Sub Update(ByVal FrontImage As Image, ByVal BackImage As Image, ByVal strMicr As String, ByVal strOcr As String)
        If (m_objMainForm.InvokeRequired) Then
            If (m_UpdateCallback Is Nothing) Then
                m_UpdateCallback = New UpdateCallback(AddressOf Update)
            End If
            m_IAar = m_objMainForm.BeginInvoke(m_UpdateCallback, New Object() {FrontImage, BackImage, strMicr, strOcr})
        Else
            m_objMainForm.SetFrontImage(FrontImage)
            m_objMainForm.SetBackImage(BackImage)
            m_objMainForm.SetMicr(strMicr)
            m_objMainForm.SetOcr(strOcr)
        End If
    End Sub

    ' this method is return transaction number
    Public Function GetTransactionNumber() As Integer
        Return m_objmfDevice.TransactionNumber
    End Function

    ' settings endorse text
    Private Sub ElectricEndorseText()
        Dim mfDecorate As MFTrueType = Nothing
        Dim objFont As System.Drawing.Font
        m_objmfDevice.ResetLastError()

        mfDecorate = New MFTrueType
        mfDecorate.Color = PrintColor.MF_PRINT_BLACK
        objFont = New System.Drawing.Font("Arial", 30)
        mfDecorate.Font = objFont
        ' set position
        If (CheckResponse(m_objmfDevice.SetPrintPosition(10, 60), "BiSetPrintPosition(10, 60)")) Then
            Return
        End If
        m_errResultCode = m_objmfDevice.PrintText("Transaction#:<00000000>", mfDecorate)
        If (CheckResponse(m_errResultCode, "BiPrintText")) Then
            Return
        End If
    End Sub

    ' settings endorse image
    Private Sub ElectricEndorseImage()
        ' set position
        If (CheckResponse(m_objmfDevice.SetPrintPosition(100, 30), "BiSetPrintPosition(100, 30)")) Then
            Return
        End If
        ' set size
        If (CheckResponse(m_objmfDevice.SetPrintSize(30, 30), "BiSetPrintSize(30, 30)")) Then
            Return
        End If
        If (CheckResponse(m_objmfDevice.PrintImage("Image.jpg"), "BiPrintImage(Image.jpg)")) Then
            Return
        End If
    End Sub

    ' this method is return response
    ' SUCCESS : Return to FALSE
    ' ERROR : Display the message box, return TRUE
    Private Function CheckResponse(ByVal errCode As ErrorCode, Optional ByVal str As String = "Error") As Boolean
        If Not (ErrorCode.SUCCESS.Equals(errCode)) Then
            MessageBox.Show(str, errCode.ToString(), MessageBoxButtons.OK, MessageBoxIcon.Error)
            Return True
        End If
        Return False
    End Function

    ' this method is return the  value of MF_BUZZER_HZ_XXX
    Private Function BuzzerHz(ByVal HzValue As Integer) As MfBuzzerHz
        Select Case (HzValue)
            Case 0
                Return MfBuzzerHz.MF_BUZZER_HZ_440
            Case 1
                Return MfBuzzerHz.MF_BUZZER_HZ_880
            Case 2
                Return MfBuzzerHz.MF_BUZZER_HZ_4000
            Case Else
                Return MfBuzzerHz.MF_BUZZER_HZ_440
        End Select
    End Function

    ' this method is return the  value of MF_BUZZER_COUNT_XXX
    Private Function BuzzerCount(ByVal Count As Integer) As MfBuzzerCount
        Select Case (Count)
            Case 0
                Return MfBuzzerCount.MF_BUZZER_DISABLE
            Case 1
                Return MfBuzzerCount.MF_BUZZER_COUNT_ONE
            Case 2
                Return MfBuzzerCount.MF_BUZZER_COUNT_TWO
            Case 3
                Return MfBuzzerCount.MF_BUZZER_COUNT_MAX
        End Select
    End Function

    ' settings Base structure
    Private Sub SetBaseStruct()
        m_objmfBase.BuzzerHz(MfBuzzerType.MF_BUZZER_TYPE_SUCCESS) = BuzzerHz(m_objConfig(Properties.BUZZER_SUCCESS_HZ))
        m_objmfBase.BuzzerCount(MfBuzzerType.MF_BUZZER_TYPE_SUCCESS) = BuzzerCount(m_objConfig(Properties.BUZZER_SUCCESS_COUNT))

        m_objmfBase.BuzzerHz(MfBuzzerType.MF_BUZZER_TYPE_ERROR) = BuzzerHz(m_objConfig(Properties.BUZZER_ERROR_HZ))
        m_objmfBase.BuzzerCount(MfBuzzerType.MF_BUZZER_TYPE_ERROR) = BuzzerCount(m_objConfig(Properties.BUZZER_ERROR_COUNT))

        m_objmfBase.BuzzerHz(MfBuzzerType.MF_BUZZER_TYPE_WFEED) = BuzzerHz(m_objConfig(Properties.BUZZER_WFEED_HZ))
        m_objmfBase.BuzzerCount(MfBuzzerType.MF_BUZZER_TYPE_WFEED) = BuzzerCount(m_objConfig(Properties.BUZZER_WFEED_COUNT))
    End Sub

    ' settings Process structure
    Private Sub SetProcessStruct()
        ' Activate Mode
        If (m_objConfig(Properties.CONFIRMATION)) Then
            m_objmfProcess.ActivationMode = MfActivateMode.MF_ACTIVATE_MODE_CONFIRMATION
        Else
            m_objmfProcess.ActivationMode = MfActivateMode.MF_ACTIVATE_MODE_HIGH_SPEED
        End If

        ' PaperMisInsertion
        m_objmfProcess.PaperMisInsertionErrorSelect = GetDetect(m_objConfig(Properties.MIS_INSERT_DETECT))
        m_objmfProcess.PaperMisInsertionErrorEject = GetEject(m_objConfig(Properties.MIS_INSERT_EJECT))
        m_objmfProcess.PaperMisInsertionStamp = GetStamp(m_objConfig(Properties.MIS_INSERT_STAMP))
        m_objmfProcess.PaperMisInsertionCancel = GetCancel(m_objConfig(Properties.MIS_INSERT_CANCEL))

        ' Noise
        m_objmfProcess.NoiseErrorSelect = GetDetect(m_objConfig(Properties.NOISE_DETECT))
        m_objmfProcess.NoiseErrorEject = GetEject(m_objConfig(Properties.NOISE_EJECT))
        m_objmfProcess.NoiseStamp = GetStamp(m_objConfig(Properties.NOISE_STAMP))
        m_objmfProcess.NoiseCancel = GetCancel(m_objConfig(Properties.NOISE_CANCEL))

        ' DoubleFeed
        m_objmfProcess.DoubleFeedErrorSelect = GetDetect(m_objConfig(Properties.DOUBLE_FEED_DETECT))
        m_objmfProcess.DoubleFeedErrorEject = GetEject(m_objConfig(Properties.DOUBLE_FEED_EJECT))
        m_objmfProcess.DoubleFeedStamp = GetStamp(m_objConfig(Properties.DOUBLE_FEED_STAMP))
        m_objmfProcess.DoubleFeedCancel = GetCancel(m_objConfig(Properties.DOUBLE_FEED_CANCEL))

        ' Baddata
        m_objmfProcess.BaddataCount = m_objConfig(Properties.BADDATA_COUNT)
        m_objmfProcess.BaddataErrorSelect = GetDetect(m_objConfig(Properties.BADDATA_DETECT))
        m_objmfProcess.BaddataErrorEject = GetEject(m_objConfig(Properties.BADDATA_EJECT))
        m_objmfProcess.BaddataStamp = GetStamp(m_objConfig(Properties.BADDATA_STAMP))
        m_objmfProcess.BaddataCancel = GetCancel(m_objConfig(Properties.BADDATA_CANCEL))

        ' Nodata
        m_objmfProcess.NodataErrorSelect = GetDetect(m_objConfig(Properties.NODATA_DETECT))
        m_objmfProcess.NodataErrorEject = GetEject(m_objConfig(Properties.NODATA_EJECT))
        m_objmfProcess.NodataStamp = GetStamp(m_objConfig(Properties.NODATA_STAMP))
        m_objmfProcess.NodataCancel = GetCancel(m_objConfig(Properties.NODATA_CANCEL))

        ' Near Full
        Select Case (m_objConfig(Properties.NEAR_FULL))
            Case 0
                m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_PERMIT
            Case 1
                m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_MAIN_PERMIT
            Case 2
                m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_SUB_PERMIT
            Case 3
                m_objmfProcess.NearFullSelect = MfNearFullSelect.MF_NEARFULL_NOT_PERMIT
        End Select

        ' Success Stamp
        m_objmfProcess.SuccessStamp = GetStamp(m_objConfig(Properties.SUCCESS_STAMP))
    End Sub

    ' settings Iqa structure
    Private Sub SetIqaStruct()
        m_objmfIqa.ErrorSelect = GetDetect(m_objConfig(Properties.IQA_DETECT))
        m_objmfIqa.ErrorEject = GetEject(m_objConfig(Properties.IQA_EJECT))
        m_objmfIqa.Stamp = GetStamp(m_objConfig(Properties.IQA_STAMP))
        m_objmfIqa.Cancel = GetCancel(m_objConfig(Properties.IQA_CANCEL))

        If (m_objConfig(Properties.IQA_GRAYSCALE)) Then
            m_objmfIqa.ColorDepth = ColorDepth.EPS_BI_SCN_8BIT
            m_objmfIqa.ImageFormat = Format.EPS_BI_SCN_JPEGNORMAL
        Else
            m_objmfIqa.ColorDepth = ColorDepth.EPS_BI_SCN_1BIT
            m_objmfIqa.ImageFormat = Format.EPS_BI_SCN_TIFF
        End If


        If (m_objmfIqa.ErrorSelect = MfErrorSelect.MF_ERROR_SELECT_DETECT) Then
            m_objmfIqa.Undersize = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Oversize = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Mincompressed = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Maxcompressed = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Front_rear = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Toolight = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Toodark = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Streaks = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Noise = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Focus = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Corners = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Edges = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Framing = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Skew = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Carbon = MfIqaTest.MF_IQA_TEST_ENABLE
            m_objmfIqa.Piggyback = MfIqaTest.MF_IQA_TEST_ENABLE
        End If
    End Sub

    ' setting Barcode structure
    Private Sub SetBarcodeStruct()
        m_objmfBarcodeFront.ErrorSelect = GetDetect(m_objConfig(Properties.BARCODE_FRONT_DETECT))
        m_objmfBarcodeFront.ErrorEject = GetEject(m_objConfig(Properties.BARCODE_FRONT_EJECT))
        m_objmfBarcodeFront.Stamp = GetStamp(m_objConfig(Properties.BARCODE_FRONT_STAMP))
        m_objmfBarcodeFront.Cancel = GetCancel(m_objConfig(Properties.BARCODE_FRONT_CANCEL))

        m_objmfBarcodeFront.Info(0).SymbolMask = MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODABAR Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE128 Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE39 Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_EAN_JAN Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_ITF Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_A Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_E

        m_objmfBarcodeBack.ErrorSelect = GetDetect(m_objConfig(Properties.BARCODE_BACK_DETECT))
        m_objmfBarcodeBack.ErrorEject = GetEject(m_objConfig(Properties.BARCODE_BACK_EJECT))
        m_objmfBarcodeBack.Stamp = GetStamp(m_objConfig(Properties.BARCODE_BACK_STAMP))
        m_objmfBarcodeBack.Cancel = GetCancel(m_objConfig(Properties.BARCODE_BACK_CANCEL))

        m_objmfBarcodeBack.Info(0).SymbolMask = MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODABAR Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE128 Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_CODE39 Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_EAN_JAN Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_ITF Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_A Or _
                                                    MfBarcodeSymbol.MF_BARCODE_SYMBOL_UPC_E

    End Sub

    ' MICR data is preserved
    Private Sub MicrSave(ByVal transactionNumber As Integer)
        ' Create log file name
        Dim sw As System.IO.StreamWriter = New System.IO.StreamWriter(transactionNumber.ToString() + ".Micr.txt")
        Try
            ' Values that are always printed out
            sw.WriteLine("# MICR Data Output")
            sw.WriteLine()
            sw.WriteLine("# General Transaction Information:")
            sw.WriteLine()
            sw.WriteLine("TransactionNumber = " + transactionNumber.ToString())
            ' MICR
            If m_objConfig(Properties.MICR) Then
                m_objmfMicr.Parsing = True
                ' Acquire MICR text
                m_objmfMicr.MicrOcrSelect = MfMicrType.MF_MICR_USE_MICR
                ' Get the text acquired
                m_objmfDevice.GetMicrText(transactionNumber, m_objmfMicr)
                sw.WriteLine("Result = " + m_objmfMicr.Ret.ToString())
                sw.WriteLine("Status = " + m_objmfMicr.Status.ToString())
                sw.WriteLine("Detail = " + m_objmfMicr.Detail.ToString())
                sw.WriteLine()
                sw.WriteLine()
                sw.WriteLine("# Magnetic Character Results:")
                sw.WriteLine()
                sw.WriteLine("Raw Data       = " + m_objmfMicr.MicrStr)
                sw.WriteLine("Account Number = " + m_objmfMicr.AccountNumber)
                sw.WriteLine("Amount         = " + m_objmfMicr.Amount)
                sw.WriteLine("Bank Number    = " + m_objmfMicr.BankNumber)
                sw.WriteLine("Serial Number  = " + m_objmfMicr.SerialNumber)
                sw.WriteLine("EPC Property   = " + m_objmfMicr.EPC)
                sw.WriteLine("Transit Number = " + m_objmfMicr.TransitNumber)
                sw.WriteLine("Check Type     = " + m_objmfMicr.CheckType.ToString())
                sw.WriteLine("Country Code   = " + m_objmfMicr.CountryCode.ToString())
            End If
            If m_objConfig(Properties.OCR_AB) Then
                ' OCR AB
                Dim index As Integer
                Dim min As Integer = 100
                Dim max As Integer = 0
                Dim sum As Integer = 0

                Dim mfOcrAb As MFOcrAb = New MFOcrAb
                sw.WriteLine()
                sw.WriteLine()
                ' Font designation
                Select Case (m_objConfig(Properties.OCR_AB_FONT))
                    Case 0
                        sw.WriteLine("# OCR A character probabilities:")
                        mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRA_ALPHANUM
                    Case 1
                        sw.WriteLine("# OCR B character probabilities:")
                        mfOcrAb.OcrType = MfOcrType.MF_OCR_FONT_OCRB_ALPHANUM
                End Select

                mfOcrAb.Direction = MfDirection.MF_OCR_LEFTRIGHT
                mfOcrAb.StartX = Convert.ToUInt16(OCR_AB_START_X)
                mfOcrAb.StartY = Convert.ToUInt16(OCR_AB_START_Y)
                mfOcrAb.EndX = Convert.ToUInt16(OCR_AB_END_X)
                mfOcrAb.EndY = Convert.ToUInt16(OCR_AB_END_Y)
                mfOcrAb.SpeceHandling = MfSpeceHandling.OCR_SPACE_ENABLE

                ' select front side image
                m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)

                ' Acquire OCR_AB text
                m_errResultCode = m_objmfDevice.GetOcrABText(Convert.ToUInt32(transactionNumber), OcrImageSource.OCR_SOURCE_TRANSACTION_NUMBER, "", mfOcrAb)
                sw.WriteLine("")
                sw.WriteLine("OcrAbResult = " + m_errResultCode.ToString())

                ' Acquire reliability percentage (maximum value/minimum value)
                For index = 0 To mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length - 1
                    sum = sum + mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index)
                    If (min > mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index)) Then
                        min = mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index)
                    End If
                    If (max < mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index)) Then
                        max = mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index)
                    End If
                Next
                If Not (index = 0) Then
                    Dim average As Double = sum / mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length

                    sw.WriteLine()
                    sw.WriteLine("OcrString = " + mfOcrAb.OcrReliableInfo.FirstSelectString)
                    sw.WriteLine("NumberOfCharacters = " + mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length.ToString())
                    sw.WriteLine("HighestPercentage  = " + max.ToString())
                    sw.WriteLine("LowestPercentage   = " + min.ToString())
                    sw.WriteLine("AveragePercentage  = " + average.ToString("G4"))
                    sw.WriteLine()
                    ' Output the OCR_AB Text and reliability 
                    For index = 0 To mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length - 1
                        If (index < mfOcrAb.OcrReliableInfo.FirstSelectPercentage.Length) Then
                            sw.WriteLine("OcrPrimaryCharacter" + index.ToString() + "    = '" + mfOcrAb.OcrReliableInfo.FirstSelectString.Chars(index) + "'")
                            sw.WriteLine("OcrPrimaryPercentage" + index.ToString() + "   = " + mfOcrAb.OcrReliableInfo.FirstSelectPercentage(index).ToString())
                        End If
                        If (index < mfOcrAb.OcrReliableInfo.SecondSelectPercentage.Length) Then
                            sw.WriteLine("OcrSecondaryCharacter" + index.ToString() + "  = '" + mfOcrAb.OcrReliableInfo.SecondSelectString.Chars(index) + "'")
                            sw.WriteLine("OcrSecondaryPercentage" + index.ToString() + " = " + mfOcrAb.OcrReliableInfo.SecondSelectPercentage(index).ToString())
                        End If
                    Next
                End If
            End If
        Finally
            sw.Close()
        End Try
    End Sub

    Private Sub DisplayBarcodeData(ByVal transactionNumber As Integer)
        Dim strBarcodeData As String = ""
        Dim cData As Char() = Nothing
        Dim objData As MFBarcode.BarcodeData = Nothing
        Dim nCount As Integer = 0

        If (m_objConfig(Properties.BARCODE_FRONT_DETECT)) Then
            ' select front side
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_FRONT)
            m_errResultCode = m_objmfDevice.GetBarcodeData(transactionNumber, m_objmfBarcodeFront)
            If Not (m_objmfBarcodeFront.Data Is Nothing) Then
                ' convert string
                For Each objData In m_objmfBarcodeFront.Data
                    cData = New Char(objData.Data.Length - 1) {}
                    For nCount = 0 To objData.Data.Length - 1
                        If objData.Data(nCount) <> 0 Then
                            cData(nCount) = System.Convert.ToChar(objData.Data(nCount))
                        Else
                            cData(nCount) = " "
                        End If
                    Next
                    strBarcodeData = strBarcodeData + New String(cData)
                    strBarcodeData = strBarcodeData + " "
                Next
            End If
            If (m_objmfBarcodeFront.Ret <> ErrorCode.SUCCESS) Then
                strBarcodeData += "(" + m_objmfBarcodeFront.Ret.ToString() + ")"
            End If
        End If
        m_objMainForm.SetBarcodeFront(strBarcodeData)
        strBarcodeData = ""

        If (m_objConfig(Properties.BARCODE_BACK_DETECT)) Then
            ' select back side
            m_errResultCode = m_objmfDevice.SCNSelectScanFace(ScanSide.MF_SCAN_FACE_BACK)
            m_errResultCode = m_objmfDevice.GetBarcodeData(transactionNumber, m_objmfBarcodeBack)
            If Not (m_objmfBarcodeBack.Data Is Nothing) Then
                ' convert string
                For Each objData In m_objmfBarcodeBack.Data
                    cData = New Char(objData.Data.Length - 1) {}
                    For nCount = 0 To objData.Data.Length - 1
                        If objData.Data(nCount) <> 0 Then
                            cData(nCount) = System.Convert.ToChar(objData.Data(nCount))
                        Else
                            cData(nCount) = " "
                        End If
                    Next
                    strBarcodeData = strBarcodeData + New String(cData)
                    strBarcodeData = strBarcodeData + " "
                Next
            End If
            If (m_objmfBarcodeBack.Ret <> ErrorCode.SUCCESS) Then
                strBarcodeData += "(" + m_objmfBarcodeBack.Ret.ToString() + ")"
            End If
        End If
        m_objMainForm.SetBarcodeBack(strBarcodeData)
    End Sub

    ' this method is return the  value of ErrorSelect 
    Private Function GetDetect(ByVal Detect As Boolean) As Integer
        If (Detect) Then
            Return MfErrorSelect.MF_ERROR_SELECT_DETECT
        Else
            Return MfErrorSelect.MF_ERROR_SELECT_NODETECT
        End If
    End Function

    ' this method is return the  value of ErrorEject
    Private Function GetEject(ByVal Eject As Integer) As Integer
        Select Case (Eject)
            Case 0
                Return MfEject.MF_EJECT_MAIN_POCKET
            Case 1
                Return MfEject.MF_EJECT_SUB_POCKET
            Case 2
                Return MfEject.MF_EJECT_NOEJECT
        End Select
    End Function

    ' this method is return the  value of Stamp
    Private ReadOnly Property GetStamp(ByVal Stamp As Boolean) As Integer
        Get
            If (Stamp) Then
                Return MfStamp.MF_STAMP_ENABLE
            Else
                Return MfStamp.MF_STAMP_DISABLE
            End If
        End Get
    End Property

    ' this method is return the  value of Cancel
    Private ReadOnly Property GetCancel(ByVal Cancel As Boolean) As Integer
        Get
            If (Cancel) Then
                Return MfCancel.MF_CANCEL_ENABLE
            Else
                Return MfCancel.MF_CANCEL_DISABLE
            End If
        End Get
    End Property

    ' Stop buzzer during ringing
    Public Sub StopBuzzer()
        If (CheckResponse(m_objmfDevice.RingBuzzer(0, 0, Convert.ToUInt16(0), Convert.ToUInt16(0)), _
                        "RingBuzzer(0, 0, 0, 0)")) Then
        End If
    End Sub

    ' Acquire status
    Public Function GetStatus() As Integer
        Return m_objmfDevice.Status()
    End Function

    ' Called when DeviceStatus status is sent
    Private Sub StatusCallBack(ByVal Status As ASB) Handles m_objmfDevice.StatusCallback
        ' Set the status to DeviceStatus Form
        m_objMainForm.SetDeviceStatus(Status)
    End Sub

    ' Execute cleaning
    Public Sub Cleaning()
        Dim Err As Boolean
        Err = CheckResponse(m_objmfDevice.MICRCleaning(), "MICRCleaning(Handle)")
    End Sub

    ' Get error occured
    Public Function GetErrorOccured() As String
        Return m_errErrorOccured.ToString()
    End Function

    Public Function GetStartWaitTime() As TimeSpan
        Dim startWaitTime As UInt32
        startWaitTime = m_objmfProcess.StartWaitTime
        If startWaitTime.CompareTo(Convert.ToUInt32(6400)) > 0 Then
            startWaitTime = Convert.ToUInt32(6400)
        End If
        Return TimeSpan.FromMilliseconds(Convert.ToDouble(startWaitTime))
    End Function
End Class
